{{
'-------------------------------------------------------------------------------
'
' Catalina_XMM - Implements an External Memory Model (XMM) Kernel intended
'                for use by the Catalina Code Generator backend for LCC
'
' This kernel has no internal floating point support - the
' floating point routines are sent off to the Float32A plugin.
'
' Version 1.0 - initial version - by Ross Higson
' Version 1.1 - modified for extended addressing
' Version 1.2 - this is now the actual kernel used by XMM programs
' Version 1.3 - Add Morpheus Support
' Version 2.5 - Add Pullmoll's new f_d32s routine, move debug vectors,
'               add RamBlade support and minor tweaks to XMM code. 
'
' This program incorporates XMM software derived from:
'
'    Cluso's TriBladeProp Blade #2 Driver v0.201
'
'    HX512SRAM_ASM_DRV_101.spin by Andre' LaMothe, 
'       (C) 2007 Nurve Networks LLC
'
'    Morpheus System Architecture and Developer's Guide v0.90
'       (C) 2009 William Henning
'
'------------------------------------------------------------------------------
'
'    Copyright 2009 Ross Higson
'
'    The portion of this file identified as the XMM Kernel is part of the 
'    Catalina Target Package.
'
'    The Catalina Target Package is free software: you can redistribute 
'    it and/or modify it under the terms of the GNU Lesser General Public 
'    License as published by the Free Software Foundation, either version 
'    3 of the License, or (at your option) any later version.
'
'    The Catalina Target Package is distributed in the hope that it will
'    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
'    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
'    See the GNU Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public 
'    License along with the Catalina Target Package.  If not, see 
'    <http://www.gnu.org/licenses/>.
'
'------------------------------------------------------------------------------
}}

CON

' Float32 constants:
SignFlag      = $1
ZeroFlag      = $2
NaNFlag       = $8

FAddCmd       = 1 << 24       ' Send this command to Float32_A
FSubCmd       = 2 << 24       ' Send this command to Float32_A
FMulCmd       = 3 << 24       ' Send this command to Float32_A
FDivCmd       = 4 << 24       ' Send this command to Float32_A
FFloatCmd     = 5 << 24       ' Implemented in Float32_A and Float32_B
FTruncCmd     = 6 << 24       ' Implemented in Float32_A and Float32_B
FCmpCmd       = 9 << 24       ' Implemented in Float32_A and Float32_B
'
LMM_FLA    = common#LMM_FLA
'
FLOAT_XFER = common#FLOAT_XFER
'
XMM_BASE_ADDRESS = common#XMM_BASE_ADDRESS

CON

' HX512 XMM Constants (for Hybrid or Hydra only):

HX_DBUS_BIT_SHIFT    = 16
HX_CTRL_BIT_SHIFT    = 1
HX_STROBE_BIT_SHIFT  = 30

' commands for controller, pre-shifted to left 1-bit, so we don't have to do it
' during runtime, small enough to fit in constants
HX_CMD_WRITE_SHIFT   = %00000_00_0
HX_CMD_READ_SHIFT    = %00000_01_0
HX_CMD_LOADLO_SHIFT  = %00000_10_0
HX_CMD_LOADHI_SHIFT  = %00000_11_0

' sram commands
HX_CMD_WRITE   = %000000_00
HX_CMD_READ    = %000000_01
HX_CMD_LOADLO  = %000000_10
HX_CMD_LOADHI  = %000000_11
'
' define SHARED_XMMD on platforms that must share the XMM RAM pins with other devices:
'
#ifdef TRIBLADEPROP
#define SHARED_XMM
#elseifdef RAMBLADE
#define SHARED_XMM
#elseifdef C3
#define SHARED_XMM
#endif
OBJ

  common :  "Catalina_Common"
  
PUB Entry_Addr : addr

   addr := @entry ' Kernel Entry Address

PUB Start (registry) : cog | i

  cog := cognew(@entry, registry)

DAT
 org 0
'
entry
'
'--------------------- Start of XMM Kernel ------------------------------------
'
   jmp #INIT            '$00
'
'------------------------------------------------------------------------------
   jmp #INIT            '$01 remove this line when using POD, since it
                        '    adds a jump instruction before the program
'------------------------------------------------------------------------------
'
INIT    jmp #lmm_init   '$02
LODL    jmp #load_l     '$03
LODA    jmp #load_a     '$04
LODF    jmp #fp_ind     '$05
PSHL    jmp #push_l     '$06
PSHB    jmp #push_b     '$07
CPYB    jmp #copy_b     '$08
NEWF    jmp #new_fp     '$09
RETF    jmp #fp_ret     '$0a
CALA    jmp #f_call     '$0b
RETN    jmp #f_ret      '$0c
CALI    jmp #f_cali     '$0d
JMPA    jmp #f_jump     '$0e
JMPI    jmp #f_jmpi     '$0f
DIVS    jmp #f_d32s     '$10
DIVU    jmp #f_d32u     '$11
MULT    jmp #f_m32      '$12
BR_Z    jmp #bra_z      '$13
BRNZ    jmp #bra_nz     '$14
BRAE    jmp #bra_ae     '$15
BR_A    jmp #bra_a      '$16
BRBE    jmp #bra_be     '$17
BR_B    jmp #bra_b      '$18
SYSP    jmp #plugin     '$19
PSHA    jmp #push_a     '$1a
FADD    jmp #flt_add    '$1b
FSUB    jmp #flt_sub    '$1c
FMUL    jmp #flt_mul    '$1d
FDIV    jmp #flt_div    '$1e
FCMP    jmp #flt_cmp    '$1f
FLIN    jmp #flt_int    '$20
INFL    jmp #int_flt    '$21
PSHM    jmp #push_m     '$22
POPM    jmp #pop_m      '$23
PSHF    jmp #push_i     '$24
RLNG    jmp #rd_long    '$25
RWRD    jmp #rd_word    '$26
RBYT    jmp #rd_byte    '$27
WLNG    jmp #wr_long    '$28
WWRD    jmp #wr_word    '$29
WBYT    jmp #wr_byte    '$2a
'                           
PC      long 0          '$2b
SP      long 0          '$2c
FP      long 0          '$2d
RI      long 0          '$2e
BC      long 0          '$2f
BA      long 0          '$30
BZ      long 0          '$31
CS      long 0          '$32
'
r0      long 0          '$33
r1      long 0          '$34
r2      long 0          '$35
r3      long 0          '$36
r4      long 0          '$37
r5      long 0          '$38
r6      long 0          '$39
r7      long 0          '$3a
r8      long 0          '$3b
r9      long 0          '$3c
r10     long 0          '$3d
r11     long 0          '$3e
r12     long 0          '$3f
r13     long 0          '$40
r14     long 0          '$41
r15     long 0          '$42
r16     long 0          '$43
r17     long 0          '$44
r18     long 0          '$45
r19     long 0          '$46
r20     long 0          '$47
r21     long 0          '$48
r22     long 0          '$49
r23     long 0          '$4a
'
Bit31   long  $80000000 '$4b
all_1s  long  $ffffffff '$4c
cviu_m1 long  $000000ff '$4d
cviu_m2 long  $0000ffff '$4e
top8    long  $ff000000 '$4f   ' top 8 bits bitmask
low24   long  $00ffffff '$50   ' low 24 bits bitmask
'
init_BZ long  $00000000 '$51   ' end of code / start of heap
init_PC long  $00000000 '$52   ' the initial PC
'
' lmm_init : initialize VM - note that this code may be overlaid by 
'            26 longs of debug code once initialization is complete  
'
lmm_init
        cogid   r0              '1 convert ...
        shl     r0,#2           '2 ... my cog id ...
        add     r0,par          '3 ... to my registration addr
        rdlong  r0,r0           '4 get my request block addr
wait
        rdlong  BA,r0 wz        '5 Wait till we are given the base address ...
  if_z  jmp     #wait           '6 .. (only used during initialization)
        add     r0,#4           '7 Load our initial SP ...
        rdlong  SP,r0           '8 ... from the request block
        mov     BZ,#init_BZ<<2+8 '9 calculate ... 
        add     BZ,BA           '10 ... pointer to initial BZ
        mov     PC,BZ           '11 load ...
        add     PC,#4           '12 ... initial ...
        rdlong  PC,PC           '13 ... PC and ...
        rdlong  BZ,BZ           '14 ... BZ and ...
        call    #XMM_Activate   '15 Initialize XMM hardware
        rdlong  CS,#$10+init_PC<<2+8+8 '16 get code segment from RAM
        sub     PC,CS           '17 correct PC for XMM
        jmp     #LMM_loop       '18 we can now start executing LMM code
        nop                     '19                     
        nop                     '20                     
        nop                     '21                     
        nop                     '22                     
        nop                     '23                     
        nop                     '24                     
        nop                     '25                     
        nop                     '26                     
        nop                     '27                     
        nop                     '28                     
'
' LMM_next - increment the PC then execute the instruction pointed to by the new PC
'        
LMM_next
        add    PC,#4
'
LMM_loop
        movd   XMM_Dst,#LMM_1
        call   #XMM_ReadInstr
LMM_1
        nop
'
' No point in unrolling the loop since XMM accesses take too long to hit the Propeller 'sweet spot'
' 
{
        movd   XMM_Dst,#LMM_2
        call   #XMM_ReadInstr
LMM_2
        nop
        movd   XMM_Dst,#LMM_3
        call   #XMM_ReadInstr
LMM_3
        nop
        movd   XMM_Dst,#LMM_4
        call   #XMM_ReadInstr
LMM_4
        nop
}
        jmp    #LMM_loop

'
'load_l - load the long stored at the PC into RI, incrementing the PC
'         NOTE: For the XMM kernel, load_l and load_a are identical
'
load_l
'
'load_a - load the long address at the PC into RI, incrementing the PC
'         NOTE: For the XMM kernel, load_l and load_a are identical
'
load_a
       call   #XMM_ReadRI   ' load the address
       jmp    #LMM_loop     ' execute next instruction
'
' fp_ind - load the FP into RI, then add the long at the PC to result, incrementing the PC
'
fp_ind
       call   #XMM_ReadRI   ' load the long value
       adds   RI,FP         ' add the frame pointer
       jmp    #LMM_loop     ' execute next instruction
'
' push_a - push a value whose address is specified indirectly in a global
'          variable onto the stack
'
push_a
       call   #XMM_ReadRI   ' load the address
#ifdef LARGE
       cmp    RI,CS wc
  if_c rdlong RI,RI
  if_c jmp    #push_l
       mov    XMM_Len,#4
       movd   XMM_Dst,#RI   ' dest register is RI
       mov    XMM_Addr,RI   ' address is in RI
       sub    XMM_Addr,CS   ' correct XMM address
       call   #XMM_ReadMult ' read the bytes
       jmp    #push_l       ' ... the value just read
#else
       jmp    #push_x       
#endif
'
' push_i - push a value whose address is specified indirectly in a frame variable
'          onto the stack
'
' push_l - push a long value onto the stack, decrementing the SP
'
push_i
       call   #XMM_ReadRI   ' load the long value
       adds   RI,FP         ' add the frame pointer
push_x
       rdlong RI,RI         ' read the value at that address
push_l 
       sub    SP,#4         ' decrement SP
       wrlong RI,SP         ' save value on stack
       jmp    #LMM_loop     ' execute next instruction
'
' push_m - push many registers (specified in RI) onto the stack,
' decrementing the SP before each one
'
push_m
       call   #XMM_ReadRI   ' load the long value specifying the registers to push
       movd   :push_x,#r6   ' start with ...
       shr    RI,#6         ' ... r6
       mov    t1,#18        ' we will push up to 18 registers (up to R24)
:push_nxt
       shr    RI,#1 wc,wz   ' save rx?
 if_c  sub    SP,#4         ' if so ...
:push_x                     ' ... save ...
 if_c  wrlong 0,SP          ' ... the register
 if_z  jmp    #LMM_loop     ' no more to save
       add    :push_x,dlsb  ' point to next register
       djnz   t1,#:push_nxt ' continue till all 24 registers checked
       jmp    #LMM_loop     ' execute next instruction
        
'
' pop_m - pop many registers (specified in RI) from the stack,
' incremening the SP after each one
'
'
pop_m
       call   #XMM_ReadRI   ' load the long value specifying the registers to push
       movd   :pop_x,#r23   ' start with ...
       shl    RI,#(32-24)   ' ... r23         
       mov    t1,#18        ' we will pop up to 18 registers (down to r6)
:pop_nxt
       shl    RI,#1 wc,wz   ' load rx?
:pop_x                      ' if so ...
 if_c  rdlong 0,SP          ' ... load ...
 if_c  add    SP,#4         ' ... the register
 if_z  jmp    #LMM_loop
       sub    :pop_x,dlsb   ' point to previous register
       djnz   t1,#:pop_nxt  ' continue till all 24 registers checked
       jmp    #LMM_loop     ' execute next instruction

'
' push_b - push a structure (size in bytes at the PC) pointed to by R0
'          onto the stack, decrementing the SP. Note that the source
'          and destination data MUST be long aligned - but this is true
'          for all normal structures.
'
push_b
       call   #XMM_ReadBC   ' load the byte count from the PC
       add    BC,#3         ' round up BC ...
       andn   BC,#3         ' ... to be a multiple of 4
       sub    SP,BC         ' decrement SP by rounded up size
       mov    t1,R0         ' source is in R0
       mov    t2,SP         ' destination is SP
       jmp    #copy_longs   ' do the copy 
'
' copy_b - copy a structure (size in bytes at the PC)
'          from the address in R1 to the address in R0
'
copy_b
       call   #XMM_ReadBC   ' load the byte count from the PC
       add    BC,#3         ' round up BC ...
       andn   BC,#3         ' ... to be a multiple of 4
       mov    t1,R1         ' source is in R1
       mov    t2,R0         ' destination is in R0
' 
copy_longs
       tjz     BC,#LMM_Loop    ' no more to copy
#ifdef LARGE
       cmp     t1,CS wc        ' large mode - is src Hub Addr?
  if_c rdlong  t3,t1           ' yes - read HUB RAM to t3 ...
  if_c jmp     #:write_long    ' ... and then do write
       movd    XMM_Dst,#t3     ' no - read from XMM to t3 ...
       mov     XMM_Addr,t1     ' ... at address in t1
       sub     XMM_Addr,CS     ' ... XMM address
       call    #XMM_ReadLong   ' read from XMM to t3
:write_long
       cmp     t2,CS wc        ' large mode - is dst Hub Addr?
  if_c wrlong  t3,t2           ' yes - write t3 to Hub RAM ...
  if_c jmp     #:copy_next     ' ... and do next
       movs    XMM_Src,#t3     ' no - write to XMM from t3 ...
       mov     XMM_Addr,t2     ' ... at address in t2
       sub     XMM_Addr,CS     ' ... XMM address
       call    #XMM_WriteLong  ' write the long
#else
       rdlong  t3,t1           ' small mode - src and dst ...
       wrlong  t3,t2           ' ... must both be Hub Addr
#endif
:copy_next
       add     t1,#4           ' increment source
       add     t2,#4           ' increment destination
       sub     BC,#4           ' decrement count ...
       jmp     #copy_longs     ' ... and keep copying
'
' new_fp - save current frame pointer and set up a new frame pointer
' also calculate what SP was before data was pushed and save it in BC
'
new_fp
       sub    SP,#4         ' decrement the stack pointer
       wrlong FP,SP         ' save FP to stack
       mov    FP,SP         ' set up new FP
       add    BC,#8         ' calculate what SP was ...
       add    BC,FP         ' ... before arguments were pushed
       jmp    #LMM_loop     ' execute the next instruction
'
' fp_ret - pop current frame, restore previous frame and stack pointers,
'          then restore the previous program counter
'
' f_ret -  just restore the previous program counter
'
fp_ret
       rdlong FP,SP         ' restore previous FP
       add    SP,#4         ' increment the SP
f_ret
       rdlong PC,SP         ' read the PC
       sub    PC,CS
       add    SP,#4         ' increment the SP
       jmp    #LMM_loop     ' execute the next instruction
'
' f_call - call the routine at the address pointed to by the PC (increment the PC)
'
f_call
       call   #XMM_ReadRI   ' get the address to call
'       
' f_cali - call the routine whose address is in RI
'
f_cali
       add    PC,CS         ' correct for moved code segment
       sub    SP,#4         ' decrement the SP
       wrlong PC,SP         ' save current PC to stack
       jmp    #f_jmpi       ' go to address in RI
'
' f_jump - jump to the location at the address pointed to by PC (increment the PC)
'
f_jump
       call   #XMM_ReadRI   ' get the address to jump to
'
' f_jmpi - jump to the location whose address in RI
'
f_jmpi
       mov    PC,RI         ' get the address to jump to
       sub    PC,CS         ' correct for moved code segment
       jmp    #LMM_loop     ' execute next instruction
'
'fd_32s - Signed 32 bit division
'         Divisor --> r1
'         Dividend--> r0
'         Result --> Quotient in r0
'                    Remainder in r1
'
#ifdef OLD_DIVISION

' use original division algorithm

f_d32s
        mov t1,#32
        mov t2,#0               ' t2 is temp register to hold the remainder
        mov t3,r1
        shr t3,#31 WZ           ' get the divisor sign (t3)
 if_nz  sub t2,r1               ' divisor = -divisor if < 0
 if_nz  mov r1,t2
        mov t4,r0
        shr t4,#31 WZ           ' get the dividend sign (t4)
        mov t2,#0
 if_nz  sub t2,r0                ' dividend = -dividend if < 0
 if_nz  mov r0,t2
        mov t2,#0
        xor t3,t4               ' sign of result (t3)
:up
        shl r0,#1 WC
        rcl t2,#1 WC
        cmp r1,t2 WC,WZ
 if_a   jmp #:down1
        sub t2,r1
        add r0,#1
:down1
        sub t1,#1 WZ
 if_ne  jmp #:up
        mov r1,t2
        mov t2,#0
        cmp t4,#0 WZ
 if_nz  sub t2,r1               ' adjust the sign of remainder
 if_nz  mov r1,t2
        mov t2,#0
        cmp t3,#0   WZ
 if_nz  sub t2,r0
 if_nz  mov r0,t2               ' adjust the sign of quotient
        jmp #LMM_loop

#else

' use Pullmoll's new division algorithm

f_d32s
        mov  t1,#32
        mov  t2,#0              ' tmp2 is temp register to hold the remainder
        mov  t3,r1
        mov  t4,r0
        abs  r1, r1
        abs  r0, r0
        xor  t3, t4
:loop
        shl  r0,#1 WC
        rcl  t2,#1 WC
        cmpsub t2,r1 WC,WZ
 if_be add  r0,#1
        djnz t1,#:loop
        mov  r1,t2
        mov  t4,t4 WC           ' get bit #31 into carry
        negc r1,r1              ' adjust the sign of the remainder
        mov  t3,t3 WC           ' get bit #31 into carry
        negc r0,r0              ' adjust the sign of the result
        jmp  #LMM_loop
        
#endif
        
'
'f_d32u - Unsigned 32 bit division
'         Divisor : r1
'         Dividend : r0
'         Result:
'             Quotient in r0
'             Remainder in r1

f_d32u
        mov t3,#32
        mov t4, #0
:up2
        shl r0,#1       WC
        rcl t4,#1    WC
        cmp r1,t4    WC,WZ
 if_a   jmp #:down
        sub t4,r1
        add r0,#1
:down
        sub t3, #1   WZ
 if_ne  jmp #:up2
        mov r1,t4
        jmp #LMM_loop
'
'f_m32 - multiplication
'        r0 : 1st operand (32 bit)
'        r1 : 2nd operand (32 bit)
'        Result:
'           Product in r0 (<= 32 bit)
'
f_m32
        mov t3,#0
:start
        cmp r0,#0       WZ
 if_e   jmp #:down3
        shr r0,#1       WC
 if_ae  jmp #:down2
        add t3,r1    WC
:down2
        shl r1,#1       WC
        jmp #:start
:down3
        mov r0,t3
        jmp #LMM_loop
'
' bra_xx - branch if condition is true to the address at the PC,
'          otherwise, just increment the PC by 4
'
'
bra_z
  if_z  jmp    #f_jump       ' if condition true, branch is equiv to jump
        jmp    #LMM_next     ' increment PC, execute next instruction
bra_nz
 if_nz  jmp    #f_jump       ' if condition true, branch is equiv to jump
        jmp    #LMM_next     ' increment PC, execute next instruction
bra_ae
 if_ae  jmp    #f_jump       ' if condition true, branch is equiv to jump
        jmp    #LMM_next     ' increment PC, execute next instruction
bra_a
 if_a   jmp    #f_jump       ' if condition true, branch is equiv to jump
        jmp    #LMM_next     ' increment PC, execute next instruction
bra_be
 if_be  jmp    #f_jump       ' if condition true, branch is equiv to jump
        jmp    #LMM_next     ' increment PC, execute next instruction
bra_b
 if_b   jmp    #f_jump       ' if condition true, branch is equiv to jump
        jmp    #LMM_next     ' increment PC, execute next instruction

'
' plugin - call a plugin
' On entry:
'       R3 = code ($80 + cog id, or plugin type)
'       R2 = data (optional pointer)
'
' On exit:
'       R0 = result
'
' TODO - should remember plugins once found by type!
'
plugin

#ifdef SHARED_XMM
        call    #XMM_Tristate   ' disable XMM while calling plugin
#endif

        mov     r0,r3           ' get plugin code
        test    r0,#$80 wz      ' request by type?
 if_nz  and     r0,#$7f         ' no - code is cog id, so ...
 if_nz  shl     r0,#2           ' ... multiply cog id by 4 ...
 if_nz  add     r0,par          ' ... add registry base to get registry entry
 if_z   call    #find_plugin    ' yes - find plugin type
        rdlong  r0,r0           ' get request block from registry
        test    r0,top8 wz      ' plugin registered?
 if_z   jmp     #plugerr        ' no - error -1
        and     r0,low24        ' yes - write request ...
        wrlong  r2,r0           ' ... to request block
loop    rdlong  r3,r0   wz      ' wait till ...
 if_nz  jmp     #loop           ' ... request completed
        add     r0,#4           ' get results ...
        rdlong  r0,r0           ' ... from request block
        jmp     #plugend        'done
plugerr
        neg     r0,#1           ' plugin not registered
plugend

#ifdef SHARED_XMM
        call    #XMM_Activate   ' re-enable XMM
#endif
        jmp     #LMM_loop

'
' rd_long/rd_word/rd_byte : Read long/word/byte from memory from address in RI into BC.
'
' On entry:
'    RI : address to read
' On exit:
'    BC : long/word/byte read
'
' NOTE: If we are using the LARGE address model, we accept both HUB and XMM addreses,
'       otherwise we accept only XMM addresses (because we can't tell the difference). 
'
rd_long
#ifdef LARGE
        cmp     RI,CS wc
   if_c rdlong  BC,RI
   if_c jmp     #LMM_Loop
#endif
        mov     XMM_Len,#4
        mov     t1,#0
        jmp     #rd_mult

rd_word
#ifdef LARGE
        cmp     RI,CS wc
   if_c rdword  BC,RI
   if_c jmp     #LMM_Loop
#endif
        mov     XMM_Len,#2
        mov     t1,#16
        jmp     #rd_mult

rd_byte
#ifdef LARGE
        cmp     RI,CS wc
   if_c rdbyte  BC,RI
   if_c jmp     #LMM_Loop
#endif
        mov     XMM_Len,#1
        mov     t1,#24

rd_mult
        movd    XMM_Dst,#BC     ' dest register is BC
        mov     XMM_Addr,RI     ' address is in RI
        sub     XMM_Addr,CS     ' correct XMM address
        call    #XMM_ReadMult   ' read the bytes
        ror     BC,t1
        jmp     #LMM_Loop
'
' wr_long/wr_word/wr_byte : Write long/word/byte from BC to memory at address in RI.
'
' On entry:
'    RI : address to write
'    BC : long/word/byte to write
' On exit:
'    (none)
'
' NOTE: If we are using the LARGE address model, we accept both HUB and XMM addreses,
'       otherwise we accept only XMM addresses (because we can't tell the difference). 
'
wr_long
#ifdef LARGE
        cmp     RI,CS wc
   if_c wrlong  BC,RI
   if_c jmp     #LMM_Loop
#endif
        mov     XMM_Len,#4
        jmp     #wr_mult

wr_word
#ifdef LARGE
        cmp     RI,CS wc
   if_c wrword  BC,RI
   if_c jmp     #LMM_Loop
#endif
        mov     XMM_Len,#2
        jmp     #wr_mult

wr_byte
#ifdef LARGE
        cmp     RI,CS wc
   if_c wrbyte  BC,RI
   if_c jmp     #LMM_Loop
#endif
        mov     XMM_Len,#1

wr_mult
        movs    XMM_Src,#BC     ' source register is BC
        mov     XMM_Addr,RI     ' address is in RI
        sub     XMM_Addr,CS     ' correct XMM address
        call    #XMM_WriteMult  ' write the bytes
        jmp     #LMM_Loop

'------------------------------------------------------------------------------
' Float32 Assembly language jump points
'------------------------------------------------------------------------------
flt_add
        mov     t2, #FAddCmd>>24
        jmp     #sendCmd
flt_sub
        mov     t2, #FSubCmd>>24
        jmp     #sendCmd
flt_mul
        mov     t2, #FMulCmd>>24
        jmp     #sendCmd
flt_div
        mov     t2, #FDivCmd>>24
        jmp     #sendCmd
flt_int
        mov     t2, #FFLoatCmd>>24
        jmp     #sendCmd
int_flt
        mov     t2, #FTruncCmd>>24

'------------------------------------------------------------------------------
' sendCmd - send a command to Float A for execution
' input:   t2           command
'          r0          32-bit floating point value
'          r1          32-bit floating point value
' output:  r0          32-bit floating point result
'------------------------------------------------------------------------------

sendCmd
        shl     t2, #24         ' command to send to Float32A
        mov     t1, xfer        ' get pointer to xfer block
        or      t2, t1          ' construct request

        wrlong  r0, t1          ' write first argument to xfer block
        add     t1, #4
        wrlong  r1, t1          ' write second argument to xfer block

        mov     r0,#LMM_FLA     ' plugin type we want
        call    #find_plugin    ' find the plugin
        cmp     t4,#8 wc,wz     ' ran out of plugins?
 if_ae  jmp     #sendErr        ' yes - no such plugin
        rdlong  r0,r0           ' get request block from registry
        and     r0,low24        ' no - write request ...
        wrlong  t2,r0           ' ... to request block
loop2   rdlong  t2,r0   wz      ' wait till ...
 if_nz  jmp     #loop2          ' ... request completed
        add     r0,#4           ' get results ...
        rdlong  r0,r0           ' ... from request block
        jmp     #LMM_Loop
sendErr
        mov     r0, NaN         ' return NaN on any error
        jmp     #LMM_Loop

'------------------------------------------------------------------------------
' _FCmp    set Z and C flags for r0 - r1
' changes: t1
'------------------------------------------------------------------------------
flt_cmp
        mov   t1, r0            ' compare signs
        xor   t1, r1
        and   t1, Bit31 wz
 if_z   jmp   #:cmp1            ' same, then compare magnitude

        mov   t1, r0            ' check for +0 or -0
        or    t1, r1
        andn  t1, Bit31 wz,wc
 if_z   jmp   #LMM_loop

        test  r0, Bit31 wc      ' compare signs
        jmp   #LMM_loop

:cmp1   test  r0, Bit31 wz      ' check signs
  if_nz jmp   #:cmp2
        cmp   r0, r1 wz,wc
        jmp   #LMM_loop
:cmp2   cmp   r1, r0 wz,wc      ' reverse test if negative
        jmp   #LMM_loop
'
' find_plugin : find a plugin by type
'
' On entry:
'   r0 : plugin type to find
' On exit:
'   r0 : registry address of the plugin if found
'   t1 : cog number if found, 8 if not found
'
find_plugin
        mov     t1,r0           ' save type to find
        mov     r0,par          ' point to registry
        mov     t4,#0           ' start at cog 0
:find_loop
        cmp     t1,#8 wc,wz     ' run out of plugins?
 if_ae  jmp     #:find_done     ' yes - no such plugin
        rdlong  t5,r0           ' no - check next plugin type
        shr     t5,#24          ' is it ...
        cmp     t5,t1 wz        ' ... the type what we wanted?
 if_z   jmp     #:find_done     ' yes - return this cog
        add     t4,#1           ' no ...
        add     r0,#4           ' ... check ...
        jmp     #:find_loop     ' ... next cog
:find_done
        mov     t1,t4        ' return cog we found
find_plugin_ret
        ret

'
'------------------------- Float32 constant values -----------------------------

NaN     long    $7FFF_FFFF
dlsb    long    1 << 9
xfer    long    FLOAT_XFER      ' transfer address to send command to Float32A

'------------------------ temporary local variables -----------------------------
'
t1      long    $0
t2      long    $0
t3      long    $0
t4      long    $0
t5      long    $0
'
'----------------------- Kernel-specific XMM Routines --------------------------
'
' XMM_ReadBC : Read long from XMM memory at address in PC into BC.
'
' On entry: PC (32-bit): address to read from (19-bits used)
' On exit:  BC loaded with long pointed to by PC, PC incremented by 4
'
' XMM_ReadRI : Read long from XMM memory at address in PC into RI.
'
' On entry: PC (32-bit): address to read from (19-bits used)
' On exit:  RI loaded with long pointed to by PC, PC incremented by 4
'
' XMM_ReadInstr : Read instruction from XMM memory at address in PC into a register.
'
' On entry: PC (32-bit): address to read from (19-bits used)
'           XMM_Dst    : destination of this instruction set to destination register
' On exit:  destintation register loaded with long pointed to by PC, PC incremented by 4
'
XMM_ReadBC
              movd    XMM_Dst,#BC               ' dest register is BC
              jmp     #XMM_ReadInstr                ' 
XMM_ReadRI
              movd    XMM_Dst,#RI               ' dest register is RI
                                                ' fall through to XMM_ReadInstr
XMM_ReadInstr
              mov     XMM_Addr,PC               ' source address is PC
              call    #XMM_ReadLong             ' read the instuction
              add     PC,#4                     ' increment the PC
XMM_ReadInstr_ret
XMM_ReadRI_ret
XMM_ReadBC_ret
              ret
'----------------------------- End of XMM Kernel -------------------------------

'
'=============================== XMM SUPPORT CODE ==============================
'
' The folling #defines determine which XMM functions are included - comment out
' the appropriate lines to exclude the corresponding XMM function:
'
#define NEED_XMM_READLONG
#define NEED_XMM_WRITELONG
'#define NEED_XMM_READPAGE
'#define NEED_XMM_WRITEPAGE
'#define NEED_XMM_INCADDR        ' required for XMM_ReadPage and XMM_WritePage
'#define ACTIVATE_INITS_XMM      ' the HX512 does not allow this in the Kernel
'                                    
#include "XMM.inc"
'
'============================ END OF XMM SUPPORT CODE ==========================
'
              fit       $1eb                    ' last 5 longs reserved for debug overlay vectors
'              
              org       $1eb
DEBUG_VECTORS long      0,0,0,0,0
'
              fit       $1f0                    ' max size
' 