{{
'-------------------------------------------------------------------------------
' IMPORTANT NOTE: As of Catalina 2.8, this file is deprecated. Please use the
' Catalyst loader (Catalyst_XMM_SD_Loader) instead - it contains all the
' functionality of this program plus more. This file will be retained for at
' least one more release, but new loader functionality will not be included.
'-------------------------------------------------------------------------------
'
' Catalina_XMM_SD_Loader - loads programs from SD Card
' 
' This program can read the root directory of an SPI SD Card, and then prompts
' the user to select a file to load. If that file is found, the file details
' (i,e, the file cluster list) are loaded into a known location in hub RAM.
' The idea of this is that a low level PASM load routine can then be used to
' load the program into XMM RAM and Hub RAM, using the cluster information
' stored in hub RAM, without having to decode the details of the file system.
' Note that in the process of loading, the low level loader will overwrite the
' SPIN components of this object (and all other SPIN objects).
' 
' Currently, this loader only supports FAT16, and can only load programs from 
' the root directory (these are limitations of fsrw - but once the program is 
' loaded and running, the Catalina file system can read FAT12, FAT16 or FAT32 
' SD Cards, and can read and write files from subdirectories as well as the  
' root directory).
' 
' Because this loader must load the entire list of file clusters before it can 
' load the file, the cluster size of the SD card affects the maximum size of 
' programs that can be loaded, as follows:
' 
' Cluster Size    Max Program Size
' ============    ================
' 512 bytes        64 kbyte
'   1 kbyte       128 kbyte
'   2 kbyte       256 kbyte
'   4 kbyte       512 kbyte
'   8 kbyte         1 Mbyte
'  16 kbyte         2 Mbyte
'  32 kbyte         4 Mbyte
' 
' The loader will report if the file selected cannot be loaded because it is
' too large. When the select function is called, this program will start a 
' keyboard and display driver to allow the directory to be displayed and the 
' filename to be entered. If a file is successfully loaded, this program stops
' the keyboard and screen cogs before proceeding.
' 
' NOTE: This loader does not contain any SD code - it assumes that the Catalina
'       SD Plugin has already been loaded and started by a higher level object.
'       Similarly, it assumes the SIO Plugin has been started by a higher level
'       object.
' 
' Version 1.0 - Initial version by Ross Higson.
' Version 1.1 - Don't redisplay directory on error.
'               Filename was too short to store terminating zero.
'               SpecifyFile should not reformat filename.
' Version 1.2 - Add support for multiple CPUs, and for serial I/O. If the
'               CPU specified to be loaded is not the default CPU, this
'               program now sends the entire program to the nominated CPU
'               via serial I/O.
' Version 1.3 - Same version now supports single and multiple Prop platforms.
' Version 1.4 - Add Morpheus support.
' Version 1.5 - Add backspace support, auto-extension, built-in commands and
'               command line processing - this makes it Catalyst compatible,
'               but it offers only a subset of Catalyst features.
' Version 2.5 - Add RamBlade support and minor tweaks to XMM code. 
'
' Version 2.8 - Add SMM Loader support (useful on non XMM platforms).
'             - Add support for RESERVE_COG. 
'             - This version now deprecated.
'
' Version 2.9 - Add C3 support.
'
' This program incorporates XMM software derived from:
'
'    Cluso's TriBladeProp Blade #2 Driver v0.201
'
'    HX512SRAM_ASM_DRV_101.spin by Andre' LaMothe, 
'       (C) 2007 Nurve Networks LLC
'
'    Morpheus System Architecture and Developer's Guide v0.90
'       (C) 2009 William Henning
'
'-------------------------------------------------------------------------------
}}
'
' comment these out if possible, to save space:
'
'#define NEED_SIO_READPAGE
'#define NEED_SIO_READLONG
#define NEED_SIO_WRITEPAGE
#define NEED_SIO_WRITELONG

'
'#define SLOW_XMIT              ' MAY BE REQUIRED WHEN DEBUGGING SLAVE SIO LOADER
'
#define INCLUDE_XMM_SUPPORT     ' Include XMM memory support

'#define DISPLAY_PROGRESS        ' Dsiplay serial load progress

CON

SECTOR_SIZE  = 512

#ifdef TRIBLADEPROP

#define SHARED_SD
#define INCLUDE_SIO_SUPPORT

#ifdef CPU_1
DEFAULT_CPU = 1
#else
DEFAULT_CPU = 2
#endif

#elseifdef RAMBLADE

#define SHARED_SD
DEFAULT_CPU = 0

#elseifdef C3

#define SHARED_SD
DEFAULT_CPU = 0

#elseifdef MORPHEUS

#define INCLUDE_SIO_SUPPORT
#define PROCESS_REPLIES  

DEFAULT_CPU = 1

#else

DEFAULT_CPU = 0

#endif

CLOCKFREQ      = Common#CLOCKFREQ

LASTCOG        = Common#LASTCOG

FLIST_ADDRESS  = Common#FLIST_ADDRESS
MAX_FLIST_SIZE = Common#MAX_FLIST_SIZE
LMM_FIL        = Common#LMM_FIL
LMM_HMI        = Common#LMM_HMI

XMM_BASE_ADDRESS = common#XMM_BASE_ADDRESS

CON

' HX512 XMM Constants (for Hybrid or Hydra only):

HX_DBUS_BIT_SHIFT    = 16
HX_CTRL_BIT_SHIFT    = 1
HX_STROBE_BIT_SHIFT  = 30

' commands for controller, pre-shifted to left 1-bit, so we don't have to do it
' during runtime, small enough to fit in constants
HX_CMD_WRITE_SHIFT   = %00000_00_0
HX_CMD_READ_SHIFT    = %00000_01_0
HX_CMD_LOADLO_SHIFT  = %00000_10_0
HX_CMD_LOADHI_SHIFT  = %00000_11_0

' sram commands
HX_CMD_WRITE   = %000000_00
HX_CMD_READ    = %000000_01
HX_CMD_LOADLO  = %000000_10
HX_CMD_LOADHI  = %000000_11

OBJ
  Common : "Catalina_Common"

  CogStore : "Catalina_CogStore"

#ifdef TRIBLADEPROP   
  tv  : "PC_Text"
  kbd : "PC_Keyboard"
#elseifdef MORPHEUS   
  tv  : "PC_Text"
  kbd : "PC_Keyboard"
#elseifdef RAMBLADE   
  tv  : "PC_Text"
  kbd : "PC_Keyboard"
#elseifdef DRACBLADE
  tv  : "VGA_Text"
  kbd : "comboKeyboard"
#else
  tv  : "TV_Text"
  kbd : "comboKeyboard"
#endif
  fat : "Catalina_FSRW"

#ifdef DISPLAY_PROGRESS
  HMI : "Catalina_HMI_Plugin_PC"
#endif

#ifdef RESERVE_COG
  RSV      : "Catalina_Reserved_Cog"
#endif
  
VAR
  long ui_started
  long cpu_to_load
  byte filename[16]
  byte command[1000]
  byte check[1000]

DAT

DIR_COMMAND  byte "DIR", 0
CLS_COMMAND  byte "CLS", 0
HELP_COMMAND byte "HELP", 0

DEFAULT_EXT  byte ".BIN",0

PUB Start (select) : cpu | loaded
'
' Mount the SD Card, then if select is TRUE, display the directory and allow
' the user to select a file. If select is FALSE, do nothing else - the user
' can then call either SelectFile or SpecifyFile themselves, and  build the
' list of file clusters by calling LoadClusterList.
'
  ui_started    := FALSE
  loaded        := FALSE
  cpu_to_load   := DEFAULT_CPU
  word[FLIST_ADDRESS] := $FFFF

'
' Start the cogstore for storing the command line - note this always uses cog 7
'
  CogStore.Start
  
  repeat
#ifdef C3
    ' TBD : need a new fat module for C3!!!  
    ' result := \fat.mount(Common#SD_DO_PIN, Common#SD_CLK_PIN, Common#SD_DI_PIN, Common#SD_CS_PIN)
#else
    result := \fat.mount(Common#SD_DO_PIN, Common#SD_CLK_PIN, Common#SD_DI_PIN, Common#SD_CS_PIN)
#endif    
    if (result == 0)
      if select
        'DisplayDir
        repeat
          cpu := SelectFile
          loaded := \LoadClusterList
        while loaded < 0
        return 
      else
        cpu := 0 
    else
      if select
        if not ui_started
           Start_UI
        tv.str(string("Cannot Mount SD Card",13))
        'tv.dec(result)
        tv.str(string("Check FAT16 SD Card is inserted",13,13))
        tv.str(string("Press any key to continue",13))
        kbd.getkey
        tv.out(13)
      else
        cpu := 0

PUB DisplayDir | i, j, k, name[4]
'
' Display the root directory of the SD card.
'
  if not ui_started
    Start_UI
  
  tv.str(string("               Directory", 13))
  tv.str(string("------------ ------------ ------------", 13))
  fat.opendir

  j := 0
  k := 0
  repeat while fat.nextfile(@name) == 0
    i := 0
    repeat while (i < 12 and byte[@name][i])
      tv.out(byte[@name][i++])
    repeat while (i < 13)
      tv.out(" ")
      i++
    j++
    if j == 3
      j := 0
      k++
      tv.out(13)
    if k == 8
      k := 0
      tv.str(string("Press any key to continue"))
      kbd.getkey
      tv.out(13)
      tv.str(string("------------ ------------ ------------", 13))
  tv.out(13)

PUB DisplayHelp
'
' Display some help
'
  if not ui_started
    Start_UI

  tv.str(string(13,"This program can be used to load and run"))
  tv.str(string("programs from the root directory of a", 13))
  tv.str(string("FAT16 SD card. Just enter the filename", 13))
  tv.str(string("and any parameters. If no extension is", 13))
  tv.str(string("specified then '.BIN' is assumed.", 13))
  tv.str(string("On multi-CPU systems, specify the CPU by"))
  tv.str(string("typing SHIFT+CPU (e.g. SHIFT+1 is '!')", 13,13))
  tv.str(string("The following are built-in commands:", 13))
  tv.str(string("   CLS    clear the screen", 13))
  tv.str(string("   DIR    display a directory listing", 13))
  tv.str(string("   HELP   display this help", 13))

PRI uc(c)
'
'   Compute the upper case version of a character.
'
   if ("a" =< c and c =< "z")
      return c - 32
   return c

PUB SpecifyFile (name, cpu) | i
'
' Specify a name as the filename to load (this is provided as an alternative 
' to selecting from a directory listing). The name must be 8.3 format, and if 
' it is less than 12 characters it must be zero terminated. 
'
  cpu_to_load := cpu
  repeat i from 0 to 12
    filename[i] := byte[name][i]
    if byte[name][i] == 0
      quit
      
  if ui_started
    tv.str(string("File '"))
    tv.str(@filename)
    tv.str(string("' Specified for CPU #"))
    tv.dec(cpu)
    tv.out(13)

PRI Start_UI
'
' Start a display and keyboard driver to allow file selection
'
  tv.start(Common#TV_PIN)
#ifdef TRIBLADEPROP
  kbd.start(Common#KBD_PIN, Common#KBD_PIN + 1)
#elseifdef MORPHEUS
  kbd.start(Common#KBD_PIN, Common#KBD_PIN + 1)
#elseifdef RAMBLADE
  kbd.start(Common#KBD_PIN, Common#KBD_PIN + 1)
#else
  kbd.start(Common#KBD_PIN)
#endif
  tv.str(string(13,"        SD Card Program Loader", 13))
  tv.str(string(   "        ======================", 13))
#ifdef TRIBLADEPROP
  ' On the TriBladeProp, we remind the user how to set the CPU ...
  tv.str(string(13, "SHIFT + 0, 1, 2 or 3 to set CPU",13))
#elseifdef MORPHEUS    
  ' On Morpheus, we remind the user how to set the CPU ...
  tv.str(string(13, "SHIFT + 0, 1, or 2 to set CPU",13))
#endif
  ui_started := TRUE
  
PUB SelectFile : cpu | i, c, ext
'
' Allow the user to choose a file name, and store it in filename.
' Currently, they must type the filename in - later, this could be
' changed to a menu selection. Internal commands (e.g. "DIR" are
' processed by this function, which does not return until an
' uncrecognized command (assumed to be a file to execute) is
' encountered. The '.BIN' suffix is added if it is not present. 
'
  if not ui_started
    Start_UI

  repeat
    i := 0
    c := 0
    if cpu_to_load <> 0
      tv.str(string("CPU "))
      tv.dec(cpu_to_load)
      tv.str(string("> "))
    else
      tv.str(string("> "))
    repeat 
      c := uc(kbd.getkey & $ff)
      if (i == 0) and (c == ")") and (cpu_to_load <> 0)
        tv.out(13) 
        cpu_to_load := 0
        i := -1 
        quit
      if (i == 0) and (c == "!") and (cpu_to_load <> 1)
        tv.out(13) 
        cpu_to_load := 1
        i := -1 
        quit
      if (i == 0) and (c == "@") and (cpu_to_load <> 2)
        tv.out(13) 
        cpu_to_load := 2
        i := -1 
        quit
#ifdef TRIBLADEPROP        
      if (i == 0) and (c == "#") and (cpu_to_load <> 3)
        tv.out(13) 
        cpu_to_load := 3
        i := -1
        quit
#endif
      if (c == 200)
        if i > 0
          i -= 1
          tv.out($08)
          tv.out(" ")
          tv.out($08)
      else        
        tv.out(c)
        if (c == 13) or (c == 10)
          quit
        if (i < 1000)  
          command[i++] := c

    if i => 0 
      command[i] := 0

      ' Check for built-in commands
      if strcomp(@command, @CLS_COMMAND)
        tv.out($00)
        i := -1
      elseif strcomp(@command, @DIR_COMMAND)
        DisplayDir
        i := -1
      elseif strcomp(@command, @HELP_COMMAND)
        DisplayHelp
        i := -1
      
  until i > 0

  cpu := cpu_to_load

  ' parse the command line to get the filename - if 8 characters or less with no extension,
  ' add the extension '.bin'
  i := 0
  ext := 0
  repeat
    if (i == 12) or (command[i] == 0) or (command[i] == " ")
      quit      
    filename[i] := command[i]
    if (filename[i] == ".")
      ext := 1
    i += 1

  filename[i] := 0

  if ext == 0 and i =< 8
    bytemove(@filename[i],@DEFAULT_EXT, 5)
    i += 4

  ' save the whole command line in CogStore.
  CogStore.Write(@command)
  

PUB LoadClusterList : result | chain_count, clust, i
'
' Load the cluster list of the file whose name is in filename, storing up 
' to MAX_FLIST_SIZE words starting at FLIST_ADDRESS - if there are less 
' than MAX_FLIST_SIZE longs then the list is terminated with $FFFF.
'
   chain_count := 0
   result := fat.popen(@filename, "r")
   if result == 0
     clust := fat.currentcluster
     repeat while (clust => 0) & (clust < $FFF0)
        if (chain_count < MAX_FLIST_SIZE) 
          word[FLIST_ADDRESS][chain_count] := clust
          chain_count++
          if clust => 0
            'tv.hex(clust,4)
            'tv.out(" ")
            clust := \fat.nextcluster
            fat.setcluster(clust)
          else
            if ui_started
              'tv.out(13)
              tv.str(string("Bad cluster in file chain", 13))
              tv.str(string("Check SD card format is FAT16", 13))
            abort -1
        else
          if ui_started
            'tv.out(13)
            tv.str(string("File is too large to load", 13))
            tv.str(string("Select another file", 13))
          abort -2   
   else
     if ui_started
       tv.str(string("Cannot open '"))
       tv.str(@filename)
       tv.str(string("'",13,"Check file exists", 13))
   if (chain_count < MAX_FLIST_SIZE)
      word[FLIST_ADDRESS][chain_count] := $FFFF
  if result == 0
    ' all done - stop the ui cogs if we started them
    if ui_started
      kbd.stop
      tv.stop
      ui_started := FALSE
    fat.unmount
   return result

PUB Execute(Registry_Addr, Sector_Buffer, Xfer_Buffer, Io_Block, CPU_Num, sio_cog) | Reg, Flist, Size, Shift, DStart, Sector, Xfer, CPU, Block, ALLOC, HMI_DATA, RSV_DATA
'
' Initiate the Low Level Loader (never returns)
'
  Reg      := Registry_Addr
  Flist    := FLIST_ADDRESS
  Size     := fat.currentsize
  Shift    := fat.currentshift
  DStart   := fat.currentstart
  Sector   := Sector_Buffer
  Xfer     := Xfer_Buffer
  CPU      := CPU_Num
  Block    := Io_Block

  ' For this Loader, we always start allocating down from
  ' FLIST_BUFF (instead of the REQUEST block) - this is
  ' because the load process is not yet complete and needs
  ' to use that space. This also means programs can be no
  ' larger than 31k MINUS the space the plugins allocate
  ' (or 31k if there are no plugins).
  ALLOC := common#FLIST_BUFF
 
#ifdef RESERVE_COG                          
  ' allocate data block for RSV
  ALLOC := ALLOC - RSV#BYTE_SIZE
  RSV_DATA := ALLOC 
  ' start the selected HMI Plugin (and screen drivers only)
  RSV.Start(RSV_DATA)
  rsv_begin := RSV_DATA ' don't overwrite Hub RAM block of reserved cog
  rsv_end   := RSV_DATA + RSV#BYTE_SIZE 
#endif

#ifdef DISPLAY_PROGRESS                          
  ' allocate data block for HMI
  ALLOC := ALLOC - HMI#DATASIZE
  HMI_DATA := ALLOC 
  ' start the selected HMI Plugin (and screen drivers only)
  HMI.Start(Registry_Addr, HMI_DATA, FALSE, FALSE, TRUE)
#endif

  '
  ' set the maximum we can load
  '
  max_hub_load := ALLOC
  
  ' start the low level loader
  cognew(@entry, @Reg)
  cogstop(cogid)

CON
'
' SD Plugin Constants
'
SD_Init   = 1
SD_Read   = 2
SD_Write  = 3
SD_ByteIO = 4
SD_StopIO = 5
'

DAT
        org 0

entry
        mov    r0,par                           ' point to parameters

        movd   :ld_reg,#reg_addr                ' starting ... 
        mov    r1,#8                            ' ... with reg_addr ... 
:ld_reg rdlong 0-0,r0                           ' ... read 8 parameters ...
        add    :ld_reg,d_inc                    ' ... into ...
        add    r0,#4                            ' ... consecutive ...
        djnz   r1,#:ld_reg                      ' ... registers

#ifdef INCLUDE_SIO_SUPPORT
        rdlong SIO_IO_Block,r0                  ' get address of serial io block (68 bytes)
#endif
        
        mov    clust_sects,#1                   ' calculate ...
        shl    clust_sects,clust_shift          ' ... sectors per cluster
        mov    clust_size,sect_size             ' calculate ...
        shl    clust_size,clust_shift           ' ... cluster size

#ifdef INCLUDE_XMM_SUPPORT
        call   #XMM_Activate
#endif        

#ifdef INCLUDE_SIO_SUPPORT
        cmp    cpu_no,#DEFAULT_CPU wz           ' are we loading the default CPU?
 if_z   jmp    #load_myself                     ' yes - load our own XMM and Hub RAM
'
' load a program into a different CPU using serial I/O
'        
load_another
        mov    r0,file_size
        add    r0,sect_size
        sub    r0,#1
        mov    r1,sect_size
        call   #f_d32u
        mov    sect_count,r0

        call   #SIO_WriteSync
        
        mov    src_addr,#0                      ' load from address 0 of file ...
        mov    dst_addr,cpu_no                  ' ... to offset 0 in XMM ...
        shl    dst_addr,#24                     ' ... on ...
        call   #Copy_SPI_To_SIO                 ' ... the destination CPU
        
        mov    SIO_Temp,cpu_no                  ' send ...
        shl    SIO_Temp,#24                     ' ... CPU ...
        or     SIO_Temp,SIO_EOP                 ' .. an
        call   #SIO_WriteLong                   ' ... EOP packet ...
        mov    SIO_Temp,#0                      ' ... of size ...
        call   #SIO_WriteLong                   ' ... zero

        mov    r0,cnt                           ' a small delay is required ...
        add    r0,:flushdelay                   ' ... to make sure ...
        waitcnt r0,#0                           ' .. all bytes are sent
        clkset :resetmode                       ' restart this program
                  
:flushdelay   long CLOCKFREQ / 1000             ' time to allow for flush
:resetmode    long $80

#endif

'
' load a program into this CPU
'
load_myself

        ' save the current clock freq and mode
        
        rdlong SavedFreq,#0
        rdbyte SavedMode,#4
        
        ' if the file is greater than 32k, copy data from 32k to end of file into XMM
        
        cmp    file_size,hub_size wz,wc         ' if the file size is 32k or less ...
 if_be  jmp    #done_xmm                        ' ... there is no XMM section to load                          
        mov    r0,file_size
        sub    r0,hub_size

        add    r0,sect_size
        sub    r0,#1
        mov    r1,sect_size
        call   #f_d32u
        mov    sect_count,r0
        mov    src_addr,hub_size                ' load file data from 32k ...
        mov    dst_addr,#0                      ' ... to offset 0 in XMM
        call   #Copy_SPI_To_XMM

done_xmm

        ' copy data from start of image to Hub RAM (stopping at max_hub_load)

        mov    r0,file_size
        cmp    r0,max_hub_load wc,wz
 if_be  jmp    #load_hub
        mov    r0,max_hub_load                  ' can only load to max_hub_load
load_hub    
        add    r0,sect_size
        sub    r0,#1
        mov    r1,sect_size
        call   #f_d32u
        mov    sect_count,r0
        mov    src_addr,#0                      ' load data from start of file ... 
        mov    dst_addr,#0                      ' ... to start of Hub RAM
        call   #Copy_SPI_To_RAM
'
        ' stop all cogs other than this one (up to LASTCOG), and
        ' restart this cog as a SPIN interpreter to execute the
        ' program now loaded in Hub RAM.
'
restart
        cogid   r6                              ' set our cog id
        mov     r1,#LASTCOG                     ' don't restart beyond LASTCOG
:stop_cog
        sub     r1,#1
        cmp     r1,r6 wz
 if_nz  cogstop r1
        tjnz    r1,#:stop_cog

        rdword  r3,#8                           ' Get vbase value
{
        ' calculate checksum (should be zero)
        
        mov     start_pos,#0                        
        mov     end_pos,r3
        call    #calc_checksum
        mov     hex_value,checksum
        call    #Flash_Hex
}
        ' zero hub RAM from vbase onwards. Do not zero the
        ' area used by the reserve cog (if loaded).
        
        mov     r4,r3
:zeroRam
        wrlong  Zero,r4
:incrRam        
        add     r4,#4
        cmp     r4,rsv_begin wz,wc              ' don't overwrite ...
  if_b  jmp     #:chckRam                       ' ...
        cmp     r4,rsv_end wz,wc                ' ... 
  if_b  jmp     #:incrRam                       ' ... reserved cog block
:chckRam  
        cmp     r4,hub_size wz,wc 
  if_b  jmp     #:zeroRam
  
        rdword  r2,#10                          ' Get dbase value
        sub     r2,#4
        wrlong  StackMark,r2                    ' Place stack marker at dbase
        sub     r2,#4
        wrlong  StackMark,r2
        rdlong  r2,#0
        cmp     r2,SavedFreq wz                 ' Is the clock frequency the same?
  if_ne jmp     #:changeClock
        rdbyte  r2,#4                           ' Get the clock mode
        cmp     r2,SavedMode wz                 ' If both same, just go start COG
  if_e  jmp     #:justStartUp
:changeClock
        and     r2,#$F8                         ' Force use of RCFAST clock while
        clkset  r2                              ' letting requested clock start
        mov     r2,XtalTime
:startupDelay
        djnz    r2,#:startupDelay               ' Allow 20ms@20MHz for xtal/pll to settle
        rdlong  r2,#4
        and     r2,#$FF                         ' Then switch to selected clock
        clkset  r2
:justStartUp
#ifdef INCLUDE_XMM_SUPPORT
        call   #XMM_TriState
#endif        
        mov     r0,interpreter
        coginit r0
        cogstop r6                              ' this won't be executed if we are cog 0
                
SavedFreq     long      $0
SavedMode     long      $0
StackMark     long      $FFF9FFFF               ' Two of these mark the base of the stack
Zero          long      $0
XtalTime      long      20 * 20000 / 4 / 1      ' 20ms (@20MHz, 1 inst/loop)

'-------------------------------- DEBUG routines -------------------------------
{
' calc_checksum : calculate checksum
' on entry:
'    start_pos : address of first byte to include
'    end_pos   : address of last byte to include
' on exit:
'    checksum  : checksum (including stack marker 'FF F9 FF FF FF F9 FF FF')
'    start_pos, end_pos: lost
'
calc_checksum
        mov    checksum,#$EC
:calc_loop
        rdbyte hub_byte,start_pos
        add    checksum,hub_byte
        and    checksum,#$FF
        add    start_pos,#1
        cmp    start_pos,end_pos wz,wc
  if_be jmp    #:calc_loop
calc_checksum_ret
        ret
'
start_pos      long 0
end_pos        long 1000
hub_byte       long 0
checksum       long 0
}
{
XMM_Check
        mov   XMM_Addr,CheckAddr
        movd  XMM_Dst,#CheckRslt
        call  #XMM_ReadLong
        mov   hex_value,CheckRslt
        call  #Flash_Hex
XMM_Check_ret
        ret        
        

CheckAddr     long      $4ba18 - ($8000 - $200)         ' correct for bytes removed
CheckRslt     long      $0        
}
'
{
' include some useful LED debugging routines
'
#include "debug.inc"
'
}
'-------------------------------- Utility routines -----------------------------
' Copy_SPI_To_XMM - copy a number of sectors to XMM.
'   src_addr   : address within file to start copy from.
'   dst_addr   : address to copy to.
'   sect_count : number of sectors to copy.
'
' NOTE: We start copying after an offset into the first
'       sector, but thereafter copy whole sectors.
'
' NOTE: We assume everything is LONG aligned.
'
Copy_SPI_To_XMM
#ifdef INCLUDE_XMM_SUPPORT
        tjz    sect_count,#Copy_SPI_To_XMM_ret
:Copy_loop
        mov    r0,src_addr
        call   #CalcSector
        call   #SPI_Read
        mov    XMM_Addr,dst_addr
        mov    Hub_Addr,sector_addr
        add    Hub_Addr,sect_off
        mov    XMM_Len,sect_size
        sub    XMM_Len,sect_off
        mov    r4,XMM_Len                       ' save length 
        call   #XMM_WritePage
        add    src_addr,r4
        add    dst_addr,r4
        djnz   sect_count,#:Copy_loop
#endif        
Copy_SPI_To_XMM_ret
        ret

'
' Copy_SPI_To_RAM - copy a number of sectors to RAM.
' On Entry:
'   src_addr   : address within file to start copy from.
'   dst_addr   : address to copy to.
'   sect_count : number of sectors to copy.
'
' NOTE: We start copying after an offset into the first
'       sector, but thereafter copy whole sectors.
'
' NOTE: We assume everything is LONG aligned.
'
Copy_SPI_To_RAM
        tjz    sect_count,#Copy_SPI_To_RAM_ret
:Copy_loop
        mov    r0,src_addr
        call   #CalcSector
        call   #SPI_Read
        mov    r1,sector_addr
        add    r1,sect_off
        mov    r0,sect_size
        sub    r0,sect_off
        shr    r0,#2                            ' divide by 4 to get longs
:Write_loop
        cmp    dst_addr,rsv_begin wc, wz        ' don't overwrite ...
  if_ae jmp    #Copy_SPI_To_RAM_ret             ' ... reserved cog RAM block
        rdlong r2,r1
        wrlong r2,dst_addr
        add    r1,#4
        add    dst_addr,#4
        add    src_addr,#4
        djnz   r0,#:Write_loop
        djnz   sect_count,#:Copy_loop
Copy_SPI_To_RAM_ret
        ret

#ifdef INCLUDE_SIO_SUPPORT        

' Copy_SPI_To_SIO - copy a number of sectors to SIO.
'   src_addr   : address within file to start copy from.
'   dst_addr   : address to copy to (this is passed on).
'   sect_count : number of sectors to copy.
'
' NOTE: We start copying after an offset into the first
'       sector, but thereafter copy whole sectors.
'
' NOTE: We assume everything is LONG aligned.
'
Copy_SPI_To_SIO
        tjz    sect_count,#Copy_SPI_To_SIO_ret
:Copy_loop
        mov    r0,src_addr
        call   #CalcSector
        call   #SPI_Read
        mov    SIO_Addr,dst_addr
        mov    Hub_Addr,sector_addr
        add    Hub_Addr,sect_off
        mov    SIO_Len,sect_size
        sub    SIO_Len,sect_off
        mov    r4,SIO_Len                       ' save length 
#ifdef DISPLAY_PROGRESS
        mov    r2,#"B"                          ' display ...
        call   #HMI_t_char                      ' ... progress
#endif
        call   #SIO_WritePage                   ' send the sector
#ifdef  PROCESS_REPLIES        
        mov    lrc_addr,sector_addr             ' calculate ...
        mov    lrc_size,sect_size               ' ... LRC ...
        call   #LrcBuffer                       ' ... of sector
        call   #SIO_ReadSync                    ' yes - read sync from target
        cmps   r0,#0 wz,wc                      ' on error ...
  if_b  jmp    #:Copy_loop                      ' ... try again
        call   #SIO_ReadByte                    ' read lrc value from target
        cmps   r0,#0 wz,wc                      ' on error ...
  if_b  jmp    #:Copy_loop                      ' ... try again
        cmp    r0,lrc_rslt wz                   ' was LRC ok?
  if_z  jmp    #:Lrc_ok                         ' yes - do next sector
#ifdef DISPLAY_PROGRESS
        mov    r2,#"X"                          ' display ...
        call   #HMI_t_char                      ' ... LRC failure
        call   #HMI_CrLf
#endif
        jmp    #:Copy_loop                      ' try again on error
:Lrc_ok
#ifdef DISPLAY_PROGRESS
        mov    r2,#"Y"                          ' display ...
        call   #HMI_t_char                      ' ... LRC valid
        call   #HMI_CrLf
#endif
#endif

:Copy_Next
        mov    sect_off,#0
        add    src_addr,r4
        add    dst_addr,r4
        djnz   sect_count,#:Copy_loop
Copy_SPI_To_SIO_ret
        ret
'
#endif        
'
'f_d32u - Unsigned 32 bit division
'         Divisor : r1
'         Dividend : r0
'         Result:
'             Quotient in r0
'             Remainder in r1

f_d32u
        mov    ftemp,#32
        mov    ftmp2, #0
:up2
        shl    r0,#1       WC
        rcl    ftmp2,#1    WC
        cmp    r1,ftmp2    WC,WZ
 if_a   jmp    #:down
        sub    ftmp2,r1
        add    r0,#1
:down
        sub    ftemp, #1   WZ
 if_ne  jmp    #:up2
        mov    r1,ftmp2
f_d32u_ret
        ret

{
'
'f_m32 - multiplication
'        r0 : 1st operand (32 bit)
'        r1 : 2nd operand (32 bit)
'        Result:
'           Product in r0 (<= 32 bit)
'
f_m32
        mov    ftemp,#0
:start
        cmp    r0,#0       WZ
 if_e   jmp    #:down3
        shr    r0,#1       WC
 if_ae  jmp    #:down2
        add    ftemp,r1    WC
:down2
        shl    r1,#1       WC
        jmp    #:start
:down3
        mov    r0,ftemp
f_m32_ret
        ret
}

ClearSector
        mov    r0,#0
        mov    r1,sect_size
        mov    r2,sector_addr
:Clear_loop
        wrbyte r0,r2
        add    r2,#1
        djnz   r1,#:Clear_loop
ClearSector_ret
        ret

'
' LrcBuffer - Calculate LRC of buffer
'
' On Entry
'    lrc_size = size of buffer
'    lrc_addr = address of buffer
' On exit:
'    lrc_rslt = result of LRC
'
LrcBuffer
        mov    lrc_rslt,#0
        mov    r1,lrc_size
        mov    r2,lrc_addr
:LrcBuffer_loop
        rdbyte r0,r2
        xor    lrc_rslt,r0
        add    r2,#1
        djnz   r1,#:LrcBuffer_loop
LrcBuffer_ret
        ret
'
lrc_addr      long      $0
lrc_size      long      $0
lrc_rslt      long      $0
'
' CalcSector - Calculate cluster and sector numbers and offsets
' of a given address (using the file cluster list)
'
' On entry:
'    r0 : the address
' On exit:
'    clus_num  : the cluster number containing the address
'    clus_off  : the offset into the cluster of the address
'    sect_num  : the sector number containing the address
'    sect_off  : the offset into the sector of the address
'
' NOTE: This routine does not check that the address is
'       actually WITHIN the file.
' 
CalcSector
        mov    r1,clust_size
        call   #f_d32u
        mov    clus_num,r0
        mov    clus_off,r1
        shl    r0,#1
        add    r0,list_addr
        rdword r0,r0
        shl    r0,clust_shift
        add    r0,data_region
        mov    sect_num,r0
        mov    r0,clus_off
        mov    r1,sect_size
        call   #f_d32u
        add    sect_num,r0
        mov    sect_off,r1   
CalcSector_ret
        ret

'
#ifdef INCLUDE_SIO_SUPPORT
'
'------------------------------------ SIO Routines -----------------------------------
'
#ifdef NEED_SIO_READPAGE
'
' SIO_ReadPage : Read data from SIO to HUB RAM.
' On Entry: 
'    SIO_Addr  (32-bit): source address (not used, but updated after read)
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    SIO_Len   (32-bit): number of bytes read, (max PAGE_SIZE)
'
' NOTE: data packets are:
'    4 bytes CPU + address 
'    4 bytes size (but only a max of PAGE_SIZE will actually get loaded) 
'    size bytes of data  
'
' NOTE: The top byte of the address ($nn) is the CPU number (1 .. 3).
'
' NOTE ($nn $FF $FF $FF) ($00 $00 $00 $00) indicates end of data
'
' NOTE: to maintain synchronization, all data is read even if the CPU
'       address indicates the packet is not for this CPU.

SIO_ReadPage
              call      #SIO_ReadLong           ' read ...
              mov       SIO_Addr,SIO_Temp       ' ... address
#ifdef DEBUG
              mov       r6,r0                   ' disply ... 
              mov       r2,SIO_Temp             ' ...
              call      #HMI_t_dec              ' ...
              call      #HMI_space              ' ...
              mov       r0,r6                   ' ... addresss
#endif
              call      #SIO_ReadLong           ' read ...
              mov       SIO_Len,SIO_Temp        ' ... size
#ifdef DEBUG
              mov       r6,r0                   ' display ... 
              mov       r2,SIO_Temp             ' ...
              call      #HMI_t_dec              ' ...
              call      #HMI_space              ' ... 
              mov       r0,r6                   ' ... size
#endif
              mov       SIO_Cnt1,SIO_Len        ' assume ...
              mov       SIO_Cnt2,#0             ' ... we will not discard data
              tjz       SIO_Len,#:SIO_AddrChk   ' check address if no data to read
              cmp       SIO_Len,max_page wc,wz  ' do we need to discard data?
        if_be jmp       #:SIO_RdLoop1           ' no - just read and save ...
              mov       SIO_Cnt2,SIO_Len        ' yes - calculate size of data to save ...
              sub       SIO_Cnt2,max_page       ' ... and size of data ...
              mov       SIO_Cnt1,max_page       ' ... to discard
              mov       SIO_Len,SIO_Cnt1        ' save size of data
:SIO_RdLoop1
              call      #SIO_ReadByte           ' read ...
              wrbyte    r0,Hub_Addr             ' ... and save ...
              add       Hub_Addr,#1             ' ... up to ...
              djnz      SIO_Cnt1,#:SIO_RdLoop1  ' ... max_page bytes
              tjz       SIO_Cnt2,#:SIO_AddrChk  ' if no more bytes, check address
:SIO_RdLoop2
              call      #SIO_ReadByte           ' read but discard ...
              djnz      SIO_Cnt2,#:SIO_RdLoop2  ' ... any remaining bytes
:SIO_AddrChk
              mov       r0,SIO_Addr             ' was this packet ...
              shr       r0,#24                  ' ... intended ...
              cmp       r0,cpu_no wz            ' ... for this CPU?
        if_nz jmp       #SIO_ReadPage           ' no - read another packet
              mov       r0,SIO_Addr             ' yes - remove CPU number ...
              and       r0,Low24                ' ... from ...
              mov       SIO_Addr,r0             ' ... data address
SIO_ReadPage_ret
              ret
'
#endif

#ifdef NEED_SIO_READLONG              
'
' SIO_ReadLong : Read 4 bytes from SIO to SIO_Temp
'
SIO_ReadLong
              mov       SIO_Cnt1,#4
:SIO_ReadLoop                       
              call      #SIO_ReadByte
              and       r0,#$FF                 ' mask the data to 8-bits
              andn      SIO_Temp,#$FF           ' combine byte read into SIO_Temp
              or        SIO_Temp,r0
              ror       SIO_Temp,#8
              djnz      SIO_Cnt1,#:SIO_ReadLoop
SIO_ReadLong_ret
              ret
              
#endif

'
' SIO_DataReady - check if SIO data is ready.
' On exit:
'    r1 = rx_head
'    r2 = address of rx_tail
'    r3 = rx_tail
'    Z flag set is no data ready (i.e. rx_tail = rx_head).
'
SIO_DataReady
              rdlong    r1,SIO_IO_Block         ' get rx_head
              mov       r2,SIO_IO_Block         ' get ...
              add       r2,#4                   ' ...
              rdlong    r3,r2                   ' ... rx_tail
              cmp       r1,r3 wz                ' rx_tail = rx_head?
SIO_DataReady_ret
              ret
'               
' SIO_ReadByteRaw : Read byte from SIO to r0, without byte unstuffing
'
SIO_ReadByteRaw
              mov       r0,cnt
              cmp       r0,SIO_StartTime wc
        if_nc sub       r0,SIO_StartTime
        if_c  subs      r0,SIO_StartTime
              cmp       r0,one_sec wz,wc
        if_a  jmp       #SIO_ReadByteRaw_err
              call      #SIO_DataReady          ' any SIO data available?
        if_z  jmp       #SIO_ReadByteRaw        ' no - wait
              mov       r1,SIO_IO_Block         ' yes - get received byte ...
              add       r1,#9*4                 ' ... from ... 
              add       r1,r3                   ' ... rx_buffer ...
              rdbyte    r0,r1                   ' ... [rx_tail] ...
              add       r3,#1                   ' calculate ... 
              and       r3,#$0f                 ' ... (rx_tail + 1) & $0f
              wrlong    r3,r2                   ' rx_tail := (rx_head + 1) & $0f
#ifdef DEBUG_RAW_DATA
              mov       r6,r0
              mov       r2,r0
              call      #HMI_t_dec
              call      #HMI_space
              mov       r0,r6
#endif              
              jmp       #SIO_ReadByteRaw_ret        
SIO_ReadByteRaw_err
              neg       r0,#1               
SIO_ReadByteRaw_ret
              ret
              
'
' SIO_ReadByte : Read byte from SIO to r0, unstuffing $FF $00 to just $FF
'
'          NOTE: returns -1 if the timeout expires, or -2 if a sync signal
'                (i.e. $FF CPU_no) is detected. Any other $FF $xx sequence
'                just returns as normal. 
'
SIO_ReadByte
              mov       SIO_StartTime,cnt
              test      save_data,#$100 wz      ' have we saved a byte?
        if_z  jmp       #:read_byte             ' no - read a new byte
              cmp       save_data,#$1FF wz      ' yes - was it $FF?
        if_z  jmp       #:read_byte             ' yes - read a new byte
              mov       r0,save_data            ' no - return ...
              and       r0,#$FF                 ' ... saved byte ...
              jmp       #:clear_saved_data      ' ... and clear saved data
:read_byte                      
              call      #SIO_ReadByteRaw        ' read a new byte
              cmps      r0,#0 wz,wc             ' read error?      
        if_b  jmp       #SIO_ReadByte_ret       ' yes - return the error
              cmp       save_data,#$1FF wz      ' have we saved $FF?
        if_nz jmp       #:check_for_ff          ' no - check the byte we just read
              cmp       r0,cpu_no wz            ' yes - did we just read our CPU no?
        if_z  jmp       #:read_sync             ' yes - this is a sync signal
              cmp       r0,#0 wz                ' no - did we just read $00?
        if_z  jmp       #:unstuff_ff            ' yes - unstuff the $FF
              or        r0,#$100                ' no - save ... 
              mov       save_data,r0            ' ... the byte we just read ...
              mov       r0,#$FF                 ' ... and return ...
              jmp       #SIO_ReadByte_ret       ' ... $FF instead 
:unstuff_ff        
              mov       r0,#$FF                 ' return unstuffed $FF
              jmp       #:clear_saved_data
:check_for_ff
              cmp       r0,#$ff wz              ' did we just read $FF?
        if_nz jmp       #:clear_saved_data      ' no - clear saved data and return the byte
              mov       save_data,#$1FF         ' yes - save the $FF ... 
              jmp       #:read_byte             ' ... and read another byte 
:read_sync
              neg       r0,#2                   ' return sync flag
:clear_saved_data              
              mov       save_data,#0            ' clear saved data
SIO_ReadByte_ret
              ret
'              
save_data     long      $0                      ' $100 + byte saved (e.g. $1FF if saved $FF)
'
#ifdef NEED_SIO_WRITEPAGE
'
'
' SIO_WritePage : Write bytes from Hub RAM to SIO
' On entry:
'    SIO_Addr  (32-bit): destination address (passed on)
'    Hub_Addr  (32-bit): source address
'    SIO_Len   (32-bit): number of bytes to write. 
'
SIO_WritePage
#ifdef DISPLAY_PROGRESS
              mov       r6,r0                   ' display ... 
              mov       r2,SIO_Addr             ' ...
              call      #HMI_t_dec              ' ... address ...
              call      #HMI_space              ' ... 
              mov       r2,SIO_Len              ' ...
              call      #HMI_t_dec              ' ...
              call      #HMI_space              ' ...
              mov       r0,r6                   ' ... and length
#endif
              mov       SIO_Temp,SIO_Addr       ' write ...
              call      #SIO_WriteLong          ' ... address
              mov       SIO_Temp,SIO_Len        ' write ...
              call      #SIO_WriteLong          ' ... size
              tjz       SIO_Len,#SIO_WritePage_ret ' done if size is zero 
:SIO_WriteLoop              
              rdbyte    r0,Hub_Addr             ' write ...
              call      #SIO_WriteByte          ' ... all ...
#ifdef DEBUG_SD
              rdbyte    r2,Hub_Addr             ' 
              call      #HMI_t_dec
              call      #HMI_space
#endif              
              add       Hub_Addr,#1             ' ... SIO_Len ...
              djnz      SIO_Len,#:SIO_WriteLoop ' ... bytes
SIO_WritePage_ret
              ret
'
#endif
'
' SIO_ByteDelay : Time to wait between bytes - this is currently set by
'                 trial and error, but to a fairly conservative value.
'
SIO_ByteDelay
              mov       r1,cnt                   
              add       r1,ByteDelay            
              waitcnt   r1,#0                   
SIO_ByteDelay_ret
              ret
'              
#ifdef SLOW_XMIT              
ByteDelay long CLOCKFREQ/50
#else
ByteDelay long CLOCKFREQ/6000
#endif                      
'
' SIO_WriteByte : Write byte in r0 to SIO, without byte stuffing
'
SIO_WriteByteRaw
              call      #SIO_ByteDelay          ' delay between characters
              mov       r1,SIO_IO_Block         ' get ...
              add       r1,#8                   ' ... 
              rdlong    r1,r1                   ' ... tx_head
              mov       r2,SIO_IO_Block         ' get ...
              add       r2,#12                  ' ... 
              rdlong    r2,r2                   ' ... tx_tail
              mov       r3,r1                   ' calculate
              add       r3,#1                   ' (tx_head + 1) ...
              and       r3,#$0f                 ' ... & $0f
              cmp       r3,r2 wz                ' tx_tail = (tx_head + 1) & $0f ?
        if_z  jmp       #SIO_WriteByteRaw       ' yes - wait
              mov       r2,SIO_IO_Block         ' no ...
              add       r2,#13*4                ' ... tx_buffer ...
              add       r2,r1                   ' ... [tx_head] ...
              wrbyte    r0,r2                   ' ... := r0
              mov       r1,SIO_IO_Block         ' tx_head := 
              add       r1,#8                   ' (tx_head +1) ...
              wrlong    r3,r1                   ' ... & $0f
              mov       r1,SIO_IO_Block         ' should ...
              add       r1,#6*4                 ' ... we ...
              rdlong    r1,r1                   ' ... ignore ...
              and       r1,#%1000 wz            ' ... echo ? 
        if_z  jmp       #SIO_WriteByteRaw_ret      ' no - just return
              call      #SIO_ReadByte           ' yes - recieve echo before returning              
SIO_WriteByteRaw_ret
              ret
'
' SIO_WriteByte : Write byte in r0 to SIO, performing byte stuffing
'                 by converting each $FF into $FF $00 to ensure that
'                 the sync signal of $FF $02 is never transmitted
'                 except when specifically intended.
'
SIO_WriteByte
              and       r0,#$FF
              cmp       r0,#$FF wz
        if_nz jmp       #:no_stuff
              call      #SIO_WriteByteRaw
              mov       r0,#$00
:no_stuff
              call      #SIO_WriteByteRaw
SIO_WriteByte_ret
              ret
'
' SIO_WriteSync : Write the Sync signal ($FF $01 for CPU #1, or $FF $03 for CPU #3).
'                 These sequences can never be generated accidentally because during
'                 normal sending $FF is stuffed to $FF $00
' On Entry:
'    CPU_no : CPU number (must be non zero!)
'              
SIO_WriteSync
              mov       r0,#$FF
              call      #SIO_WriteByteRaw
              mov       r0,cpu_no
              call      #SIO_WriteByteRaw
SIO_WriteSync_ret
              ret
'
' SIO_ReadSync : Wait for the Sync Signal ($FF $01 for CPU #1, or $FF $03 for CPU #3).
'                These sequences can never be generated accidentally because during
'                normal sending $FF is stuffed to $FF $00, so we can wait for this
'                without the risk of accidentally being triggered by a program being
'                sent to another CPU.
'
'                NOTE: ReadByteRaw returns failure if timeout expires
'
SIO_ReadSync
              mov       SIO_StartTime,cnt
:read_loop
              mov       r0,cnt
              cmp       r0,SIO_StartTime wc
        if_nc sub       r0,SIO_StartTime
        if_c  subs      r0,SIO_StartTime
              cmp       r0,one_sec wz,wc
        if_a  jmp       #:read_err
              call      #SIO_ReadByteRaw
              cmp       r0,#$FF wz
        if_nz jmp       #:read_loop
:check_cpu
              call      #SIO_ReadByteRaw
              cmp       r0,cpu_no wz
        if_z  jmp       #SIO_ReadSync_ret
              cmp       r0,#$FF wz
        if_z  jmp       #:check_cpu
              jmp       #:read_loop
:read_err
              neg       r0,#1               
SIO_ReadSync_ret
              ret

#ifdef NEED_SIO_WRITELONG                              
'
' SIO_WriteLong : Write 4 bytes from SIO_Temp to SIO
'
SIO_WriteLong
              mov       SIO_Cnt1,#4
:SIO_WriteLoop
              mov       r0,SIO_Temp                       
              call      #SIO_WriteByte
              ror       SIO_Temp,#8
              djnz      SIO_Cnt1,#:SIO_WriteLoop
SIO_WriteLong_ret
              ret

#endif              
'
' SIO data
'
SIO_Addr      long      $0
SIO_Len       long      $0
SIO_Cnt1      long      $0
SIO_Cnt2      long      $0
SIO_Temp      long      $0
SIO_IO_Block  long      $0                      ' address of shared I/O block
SIO_EOP       long      $00FFFFFE               ' end of page marker
SIO_StartTime long      $0
one_sec       long      Common#CLOCKFREQ
'
'---------------------------- End of SIO Routines ----------------------------
'
' end of INCLUDE_SIO_SUPPORT
'
#endif
'
'-------------------------------- HMI Routines -------------------------------
'
#ifdef DISPLAY_PROGRESS
'
' HMI_Command - send a request to HMI Plugin for execution
' On Entry
'    r1 = command
'    r2 = data
' On exit:
'    r0 = result   
'
' HMI_XferCmd - send a request to HMI Plugin for execution
'               using an xfer block to hold the data
' On Entry
'    r1 = command
'    r2 = data
' On Exit:
'    r0 = result
'
HMI_XferCmd
        wrlong r2,xfer_addr                     ' put data in xfer block
        mov    r2,xfer_addr                     ' use xfer block as data
        
HMI_Command
        shl     r1,#24          ' construct ...  
        and     r2,Low24        ' ... 
        or      r2,r1           ' ... request

        mov     r0,#LMM_HMI     ' plugin type we want 

        mov     ftemp,reg_addr  ' point to registry
        mov     ftmp2,#0        ' start at cog 0
:send1
        cmp     ftmp2,#8 wc,wz  ' run out of plugins?
 if_ae  jmp     #:error         ' yes - no such plugin
        rdlong  ftmp3,ftemp     ' no - check next plugin type
        shr     ftmp3,#24       ' is it ...
        cmp     ftmp3,r0 wz     ' ... the type what we wanted?
 if_z   jmp     #:send2         ' yes - use this plugin
        add     ftmp2,#1        ' no ...
        add     ftemp,#4        ' ... check ...
        jmp     #:send1         ' ... next cog
:send2
        mov     r0,ftmp2        ' use the cog where we found the plugin
        shl     r0,#2           ' multiply plugin (cog) id by 4 ...
        add     r0,reg_addr     ' add registry base to get registry entry
        rdlong  r0,r0           ' get request block from registry
        test    r0,top8 wz      ' plugin registered?
 if_z   jmp     #:error         ' no - return error
        and     r0,low24        ' yes - write request ...
        wrlong  r2,r0           ' ... to request block
:loop2  rdlong  r2,r0   wz      ' wait till ...
 if_nz  jmp     #:loop2         ' ... request completed
        add     r0,#4           ' get results ...
        rdlong  r0,r0           ' ... from request block
        jmp     #:done
:error
        neg     r0,#1           ' return -1 on any error
:done
        jmp     #HMI_Command_ret
HMI_Command_ret
HMI_XferCmd_ret
        ret
'
' HMI_t_dec - output a dec value (value in r2)
'
HMI_t_dec
        mov    r1,#24                         
        call   #HMI_XferCmd                   
HMI_t_dec_ret
        ret 
'
' HMI_t_hex - output a hex value (value in r2)
'
HMI_t_hex
        mov    r1,#26                         
        call   #HMI_XferCmd                   
HMI_t_hex_ret
        ret
'
' HMI_crlf - output CrLf
'
HMI_crlf
        mov    r2,#13                     
        call   #HMI_t_char                  
        mov    r2,#10                     
        call   #HMI_t_char                  
HMI_CrLf_ret
        ret
'
' HMI_space - output space
'
' HMI_t_char - output a char (char in r2)
'
HMI_Space
        mov    r2,#32                     
HMI_t_char
        mov    r1,#22                     
        call   #HMI_Command               
HMI_t_char_ret
HMI_space_ret
        ret
'
' HMI_k_clear - clear keyboard
'
HMI_k_clear
        mov   r1,#6
        call  #HMI_Command
HMI_k_clear_ret
        ret
'
' HMI_k_get - get a key
'
HMI_k_get
        mov   r1,#2
        call  #HMI_Command
HMI_k_get_ret
        ret
'
' HMI_k_ready - key ready?
'
HMI_k_ready
        mov   r1,#5
        call  #HMI_Command
        cmp   r0,#0 wz
HMI_k_ready_ret
        ret
'
' HMI_m_abs_x - get mouse abs_x data
'
HMI_m_abs_x
        mov   r1,#14
        call  #HMI_Command
HMI_m_abs_x_ret
        ret
'
' HMI_m_abs_y - get mouse abs_y data
'
HMI_m_abs_y
        mov   r1,#15
        call  #HMI_Command
HMI_m_abs_y_ret
        ret
'
' HMI_m_abs_z - get mouse abs_z data
'
HMI_m_abs_z
        mov   r1,#16
        call  #HMI_Command
HMI_m_abs_z_ret
        ret
'
' HMI_m_buttons - get mouse button state data
'
HMI_m_buttons
        mov   r1,#13
        call  #HMI_Command
HMI_m_buttons_ret
        ret
'
#endif
'-------------------------------- SPI SD Card Routines -------------------------------

' SPI_Read - send a read request to SD Plugin for execution
' On Entry
'          sector_addr sector buffer
'          sect_num    sector number
'
SPI_Read

#ifdef SHARED_SD
#ifdef INCLUDE_XMM_SUPPORT
        call    #XMM_Tristate   ' Disable XMM while using SD
#endif        
#endif

        mov     r2,#SD_Read     ' request ... 
        shl     r2,#24          ' ... read  
        mov     r1,xfer_addr    ' get pointer to xfer block 
        or      r2,r1           ' construct request

        wrlong  sector_addr,r1  ' write first argument to xfer block
        add     r1, #4
        wrlong  sect_num,r1     ' write second argument to xfer block

        mov     r0,#LMM_FIL     ' plugin type we want 

        mov     ftemp,reg_addr  ' point to registry
        mov     ftmp2,#0        ' start at cog 0
send1
        cmp     ftmp2,#8 wc,wz  ' run out of plugins?
 if_ae  jmp     #sendErr        ' yes - no such plugin
        rdlong  ftmp3,ftemp     ' no - check next plugin type
        shr     ftmp3,#24       ' is it ...
        cmp     ftmp3,r0 wz     ' ... the type what we wanted?
 if_z   jmp     #send2          ' yes - use this plugin
        add     ftmp2,#1        ' no ...
        add     ftemp,#4        ' ... check ...
        jmp     #send1          ' ... next cog
send2
        mov     r0,ftmp2        ' use the cog where we found the plugin
        shl     r0,#2           ' multiply plugin (cog) id by 4 ...
        add     r0,reg_addr     ' add registry base to get registry entry
        rdlong  r0,r0           ' get request block from registry
        test    r0,top8 wz      ' plugin registered?
 if_z   jmp     #sendErr        ' no - return error
        and     r0,low24        ' yes - write request ...
        wrlong  r2,r0           ' ... to request block
loop2   rdlong  r2,r0   wz      ' wait till ...
 if_nz  jmp     #loop2          ' ... request completed

#ifdef SHARED_SD
        mov     r2,#SD_StopIO   ' write ... 
        shl     r2,#24          ' ... stop request
        wrlong  r2,r0           ' ... to request block
loop3   rdlong  r2,r0   wz      ' wait till ...
 if_nz  jmp     #loop3          ' ... request completed
#endif

        mov     r0,#0
        jmp     #sendDone
sendErr
        neg     r0,#1           ' return -1 on any error
sendDone

#ifdef SHARED_SD
#ifdef INCLUDE_XMM_SUPPORT
        call    #XMM_Activate   ' re-enable XMM
#endif        
#endif

SPI_Read_ret
        ret
        
'
' Common variables
'
top8          long      $FF000000
low24         long      $00FFFFFF
d_inc         long      1<<9

r0            long      $0
r1            long      $0
r2            long      $0
r3            long      $0
r4            long      $0
r6            long      $0

reg_addr      long      $0
list_addr     long      $0
file_size     long      $0
clust_shift   long      $0
data_region   long      $0
sector_addr   long      $0
xfer_addr     long      $0
cpu_no        long      $0

clust_sects   long      $0
clust_size    long      $0

max_page                                        ' page size equals sector size
sect_size     long      SECTOR_SIZE

hub_size      long      $8000                   ' Hub RAM size is 32k
rsv_begin     long      $8000                   ' address of start of reserved cog block
rsv_end       long      $8000                   ' address of end of reserved cog block
max_hub_load  long      $8000                   ' address we can load up to

src_addr      long      $0
dst_addr      long      $0
sect_count    long      $0

sect_num      long      $0
sect_off      long      $0

clus_num      long      $0
clus_off      long      $0

' see http://forums.parallax.com/forums/default.aspx?f=25&m=363100
interpreter   long    ($0004 << 16) | ($F004 << 2) | %0000
'
' temporary storage used in mul & div calculations
'
ftemp         long      $0
ftmp2         long      $0
ftmp3         long      $0
'
#ifdef INCLUDE_XMM_SUPPORT
'
'=============================== XMM SUPPORT CODE ==============================
'
' The folling defines determine which XMM functions are included - comment out
' the appropriate lines to exclude the corresponding XMM function:
'
'#define NEED_XMM_READLONG
'#define NEED_XMM_WRITELONG
'#define NEED_XMM_READPAGE
#define NEED_XMM_WRITEPAGE
#define NEED_XMM_INCADDR        ' required for XMM_ReadPage and XMM_WritePage
#define ACTIVATE_INITS_XMM      ' the HX512 does not allow this in the Kernel
'                                    
#include "XMM.inc"
'
'============================ END OF XMM SUPPORT CODE ==========================
'
#else
'
HUB_Addr      long      $0
'
#endif
'
              fit       $1f0                    ' max size
'
{{
                            TERMS OF USE: MIT License                                                           

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}