{{
'-------------------------------------------------------------------------------
' XMM.inc - contains the PASM Code for all XMM access for all platforms.
'
' This file is included in the following files:
'
'       Catalina_XMM.spin
'       Catalina_XMM_SD_Loader.spin
'       Catalina_XMM_EEPROM_Loader.spin
'       Catalina_HUB_XMM_Loader.spin
'       Catalina_XMM_SIO_Loader.spin
'       Catalina_XMM_RamTest.spin
'       Catalyst_XMM_SD_Loader.spim
'
' Version 2.8 - initital version by Ross Higson.
' 
'-------------------------------------------------------------------------------
'
'    Copyright 2009 Ross Higson
'
'    This file is part of the Catalina Target Package.
'
'    The Catalina Target Package is free software: you can redistribute 
'    it and/or modify it under the terms of the GNU Lesser General Public 
'    License as published by the Free Software Foundation, either version 
'    3 of the License, or (at your option) any later version.
'
'    The Catalina Target Package is distributed in the hope that it will
'    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
'    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
'    See the GNU Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public 
'    License along with the Catalina Target Package.  If not, see 
'    <http://www.gnu.org/licenses/>.
'
'------------------------------------------------------------------------------
}}
'
#ifdef MORPHEUS

#ifdef CPU_1

'------------------- MORPHEUS CPU #1 SPI XMM Support Routines ------------------
'
' XMM_Activate : Activate the XMM bus. 
'
XMM_Activate
              muxc      XMM_Flags,#1            ' save C flag
              andn      dira,MASK               ' set all inputs (while initializing)
              andn      outa,MASK               ' all outputs zero ... 
              or        outa,CSEL               ' ... except CS
              or        dira,DIRN               ' set pin directions
              andn      outa,CSEL               ' lower CS to initiate command
              mov       outx,#%0000_0001        ' set ...
              call      #XMM_OutByte            ' ... SPI chip ...
              mov       outx,#%0100_0000        ' ... to sequential ...
              call      #XMM_OutByte            ' ... mode
              or        outa,CSEL               ' raise CS
              shr       XMM_Flags,#1 wc         ' restore C flag
XMM_Activate_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. Call this to allow other cogs to use microSD etc.
'               To activate XMM again, call XMM_Activate.  
'
XMM_Tristate
              or      outa,CSEL                 ' raise CS (deselect chip)
XMM_Tristate_ret
              ret
'
#ifdef NEED_XMM_READLONG              
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 16 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 16 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    Destination register contains bytes read from XMM.
'
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              muxc    XMM_Flags,#1              ' save C flag
              muxnz   XMM_Flags,#2              ' save Z flag
              mov     outx,#%0000_0011          ' Set Command ...
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
              add     XMM_Addr,XMM_Len          ' update XMM_Addr
              mov     XMM_Temp,#0
:ReadLoop
              call    #XMM_InByte               ' read next byte from SPI
              and     outx,#$FF
              or      XMM_Temp,outx
              ror     XMM_Temp,#8
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0,XMM_Temp              ' save the result
              or      outa,CSEL                 ' raise CS to terminate read
              shr     XMM_Flags,#1 wz,wc        ' restore C & Z flag
XMM_ReadLong_ret
XMM_ReadMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 16 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 16 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr incremented by 4.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
XMM_WriteMult
              muxc    XMM_Flags,#1              ' save C flag
              mov     outx,#%0000_0010          ' Set Command ...               
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
              add     XMM_Addr,XMM_Len          ' update XMM_Addr
XMM_Src       mov     XMM_Temp,0-0              ' save the value to write
:WriteLoop
              mov     outx,XMM_Temp
              and     outx,#$FF
              call    #XMM_OutByte              ' write a byte to SPI
              ror     XMM_Temp,#8               ' shift next byte down to prepare
              djnz    XMM_Len,#:WriteLoop       ' repeat while XMM_len <> 0
              or      outa,CSEL                 ' raise CS to terminate write
              shr     XMM_Flags,#1 wc           ' restore C flag
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, 16-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
XMM_WritePage
              muxc    XMM_Flags,#1              ' save C flag
              mov     outx,#%0000_0010          ' Set Command ...               
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
:WriteLoop
              rdbyte  outx,Hub_Addr             ' get byte from source (Hub)
              call    #XMM_OutByte              ' write byte to SPI
              add     Hub_Addr,#1               ' inc source address
              add     XMM_Addr,#1               ' inc dest address
              djnz    XMM_Len,#:WriteLoop       ' repeat while XMM_len <> 0
              or      outa,CSEL                 ' raise CS to terminate write
              shr     XMM_Flags,#1 wc           ' restore C flag
XMM_WritePage_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_READPAGE              
'
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, 16-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
XMM_ReadPage
              muxc    XMM_Flags,#1              ' save C flag
              muxnz   XMM_Flags,#2              ' save Z flag
              mov     outx,#%0000_0011          ' Set Command ...               
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
:ReadLoop
              call    #XMM_InByte               ' read byte from SPI
              wrbyte  outx,Hub_Addr             ' write byte to destination (Hub)
              add     Hub_Addr,#1               ' inc dest address
              add     XMM_Addr,#1               ' inc src address
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
              or      outa,CSEL                 ' raise CS to terminate read
              shr     XMM_Flags,#1 wz,wc        ' restore C & Z flag
XMM_ReadPage_ret
              ret
'
#endif          
'
'XMM_SetCmdAddr : Set XMM command and address (and lower CS). XMM_Activate should be called
'                 at least once before this routine is used.
'On Entry:
'   outx     : Command (2 for read, 3 for write)
'   XMM_Addr : address to set (up to 16 bits)
'On Exit:
'   XMM_Addr : preserved
'   C flag destroyed!
'
XMM_SetCmdAddr
              andn      outa,CSEL               ' lower CS to select chip
              mov       XMM_Test,#8             ' 8 bits
              rol       outx,#24                ' put first bit in MSB
:CmdLoop              
              rol       outx,#1 wc
              muxc      outa,IBIT
              or        outa,CLCK
              andn      outa,CLCK
              djnz      XMM_Test,#:CmdLoop      
              mov       XMM_Test,#16            ' 16 address bits
              rol       XMM_Addr,#16            ' put first address bit in MSB
:AddrLoop              
              rol       XMM_Addr,#1 wc
              muxc      outa,IBIT
              or        outa,CLCK
              andn      outa,CLCK
              djnz      XMM_Test,#:AddrLoop      
XMM_SetCmdAddr_ret
              ret              

'
' XMM_OutByte - send a byte to the SPI RAM
' On entry: byte in outx (lower 8 bits)
' On exit: C flag destroyed!
'
XMM_OutByte
              mov       XMM_Test,#8             ' 8 bits
              rol       outx,#24                ' put first bit in MSB
:OutLoop              
              rol       outx,#1 wc
              muxc      outa,IBIT
              or        outa,CLCK
              andn      outa,CLCK
              djnz      XMM_Test,#:OutLoop      
XMM_OutByte_ret
              ret              
'
' XMM_InByte - receive a byte from SPI RAM
' On exit: outx shifted left 8 bits, new byte in lower 8 bits
' On exit: Z flag destroyed!
'
XMM_InByte
              mov       XMM_Test,#8             ' 8 bits
:InLoop
              test      OBIT,ina wz
              shl       outx,#1
              muxnz     outx,#1
              or        outa,CLCK
              andn      outa,CLCK
              djnz      XMM_Test,#:InLoop      
XMM_InByte_ret
              ret
{
'
' XMM_GetStat - read status register (for testing only)
'              
XMM_GetStat                            
              andn      outa,CSEL               ' lower CS to initiate command
              mov       outx,#%0000_0101        ' set ...
              call      #XMM_OutByte            ' ... SPI chip ...
              mov       outx,#0
              call      #XMM_InByte            ' ... mode
              or        outa,CSEL               ' raise CS
XMM_GetStat_ret
              ret              
}
'
' XMM variables
'
outx          long      $0
'
XMM_Flags     long      $0
XMM_Len       long      $0
XMM_Temp      long      $0
XMM_Test      long      $0
XMM_Addr      long      $0
HUB_Addr      long      $0
'
'     
' Morpheus CPU #1 Hardware Access:
'
MASK          long      (|<Common#SPI_CS) | (|<Common#SPI_CLK) | (|<Common#SPI_SI) | (|<Common#SPI_SO)
DIRN          long      (|<Common#SPI_CS) | (|<Common#SPI_CLK) | (|<Common#SPI_SI)       
CSEL          long      (|<Common#SPI_CS)
OBIT          long      (|<Common#SPI_SO)
IBIT          long      (|<Common#SPI_SI)
CLCK          long      (|<Common#SPI_CLK)     
'
'--------------- End of MORPHEUS CPU #1 SPI XMM Support Routines ---------------
'
#elseifdef CPU_2
'
'--------------------- MORPHEUS CPU #2 XMM Support Routines --------------------
'
' XMM_Activate : Activate the XMM bus.
'
XMM_Activate
              neg     XMM_Page, #1              ' force XMM_SetAddr to latch page
              andn    outa, BSEL                ' set all XMM pins to low (just to be tidy) 
              andn    dira, BSEL                ' make all XMM pins inputs (float high)      
XMM_Activate_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. Call this to allow other cogs to use microSD etc.
'               To activate XMM again, call XMM_Activate.
XMM_Tristate
              andn    dira, BSEL                ' make all XMM pins inputs (float high)      
XMM_Tristate_ret
              ret
'
#ifdef NEED_XMM_READLONG              
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 22 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 24 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              call    #XMM_SetAddr              ' latch page address and set offset
              mov     XMM_Temp, #0              '
:ReadLoop
              mov     dira, OFFRMSK             ' set offset address bits to output, data bits to input, CTRL to RD (floats?)
              nop                               ' delay (required!!!)
              mov     outx, ina                 ' read ...
              and     outx, #$FF                ' ... the byte
              andn    dira, CTRL                ' turn the CTRL bits off in DIRA (they pull up)
              or      XMM_Temp, outx
              ror     XMM_Temp, #8
              add     outa, XMM_Inc             ' inc sram address (no page change)
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0, XMM_Temp             ' save the result
XMM_ReadLong_ret
XMM_ReadMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 22 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 24 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr incremented by 4.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
XMM_WriteMult
              call    #XMM_SetAddr              ' latch page address and set offset
XMM_Src       mov     XMM_Temp, 0-0             ' save the value
:WriteLoop
              mov     outx, XMM_Temp
              and     outx, #$FF
              or      outa, outx
              mov     dira, OFFWMSK             ' set offset address bits to output, data bits to output, CTRL to WR (floats?)
              nop                               ' delay (required!!!)
              andn    dira, CTRL                ' turn the CTRL bits off in DIRA (they pull up)
              nop                               ' delay (may be required!!!)
              ror     XMM_Temp, #8
              andn    outa, DATAMSK             ' clear data bits for next write
              add     outa, XMM_Inc             ' inc sram address (no page change)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
#endif
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, up to 24-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
XMM_WritePage
              call    #XMM_SetAddr              ' latch ram address
:WriteLoop
              rdbyte  outx, Hub_Addr

              or      outa, outx
              mov     dira, OFFWMSK             ' set offset address bits to output, data bits to output, CTRL to WR (floats?)
              nop                               ' delay (may be required!!!)
              andn    dira, CTRL                ' turn the CTRL bits off in DIRA (they pull up)
              nop                               ' delay (may be required!!!)
              add     Hub_Addr, #1              ' inc source address
              andn    outa, DATAMSK             ' clear data bits for next write
              call    #XMM_IncAddr              ' inc sram address  (autoinc page)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
XMM_WritePage_ret
              ret
'
#endif
'
#ifdef NEED_XMM_READPAGE
'              
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, up to 24-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
XMM_ReadPage
              call    #XMM_SetAddr              ' latch ram address
:ReadLoop
              mov     dira, OFFRMSK             ' set offset address bits to output, data bits to input, CTRL to RD (floats?)
              nop                               ' delay (required!!!)
              mov     outx, ina                 ' read byte
              andn    dira, CTRL                ' turn the CTRL bits off in DIRA (they pull up)
              wrbyte  outx, Hub_Addr            ' write byte to destination (Hub)
              add     Hub_Addr, #1              ' inc dest address
              call    #XMM_IncAddr              ' inc sram address (autoinc page)
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_ReadPage_ret
              ret
'              
#endif
'
'XMM_SetAddr : Setup initial XMM address. XMM_Activate should be called
'              at least once before this routine is used.
'On Entry:
'   XMM_Addr : address to set (up to 24 bits)
'On Exit:
'   XMM_Addr : preserved
'
' NOTE: To save Z & C flags:
'             muxc XMM_Flags, #1
'             muxnz XMM_Flags, #2
'       To restore Z & C flags:
'             shr XMM_Flags,#1 wc,wz

XMM_SetAddr
              muxnz XMM_Flags, #2               ' save Z flag
              and   XMM_Addr, low24             ' mask address to 24 bits
              add   XMM_Addr, XMM_Base          ' correct for Morpheus XMM not starting at 0
              mov   XMM_This, XMM_Addr          ' shift upper 16 bits ...
              shr   XMM_This, #8                ' ... into position (zeros end up in p26&p27, ready for latch)
              cmp   XMM_Page, XMM_This wz       ' same page ?
         if_z jmp   #:XMM_PageSet               ' yes - no need to set page
              mov   XMM_Page, XMM_This          ' no - save new page
              mov   outa, XMM_Page              ' output page address        
              muxnz dira, BSEL                  ' set latch to low
              muxz  dira,CTRL                   ' release LATCH (floats high)
:XMM_PageSet        
              mov   XMM_Temp, XMM_Addr          ' get ...
              and   XMM_Temp, #$FF              ' ... offset in page ...
              shl   XMM_Temp, #8                ' ... into position
              mov   outa, XMM_Temp 
              shr   XMM_Flags, #1 wz            ' restore Z flag
XMM_SetAddr_ret
              ret
tmp long 0              
'
#ifdef NEED_XMM_INCADDR              
'
'XMM_IncAddr : Increment XMM address, changing page if necessary. XMM_SetAddr
'              must be called at least once before this routine is used.
'On Entry:
'   XMM_Addr : 24 bits of address.
'On Exit:
'   XMM_Addr : incremented (wraps within page)
'   XMM_Page : incremented if the page number changes (and latch updated).
'
' NOTES: While slower than incrementing the memory address directly, this
'        routine automatically copes with XMM page changes.
'
XMM_IncAddr
              add     XMM_Addr,#1               ' increment sram address
              mov     XMM_Temp, XMM_Addr        ' is this ...
              and     XMM_Temp, #$FF            ' ... a new page?
              tjnz    XMM_Temp, #:XMM_IncOffs   ' no - just update offset within page
              neg     XMM_Page, #1              ' yes - force latching of new page
              sub     XMM_Addr, XMM_Base        ' correct for Morpheus XMM not starting at 0
              call    #XMM_SetAddr              ' set new page and offset
              jmp     #XMM_IncAddr_ret          ' done
:XMM_IncOffs
              shl     XMM_Temp, #8              ' move offset into position
              andn    outa, OFFAMSK             ' set offset ...
              or      outa, XMM_Temp            ' ... in page
XMM_IncAddr_ret
              ret
'
#endif              
'
' XMM variables
'
outx          long    $0
'
XMM_Len       long    $0
XMM_Temp      long    $0
XMM_This      long    $0
XMM_Addr      long    $0
XMM_Page      long    $0                        ' last page set into latch
Hub_Addr      long    $0
XMM_Inc       long    1 << 8                    ' 1 shifted for incrementing addr/page
XMM_Base      long    XMM_BASE_ADDRESS          ' physical XMM start address 
XMM_Flags     long    0                         ' place to save flags when in XMM routines

'     
' Morpheus CPU #2 Hardware Access:
'
'                                  ++++ ++++ ++++ ++++ = A24 .. A8 (latched)
'                                  |||| |||| |||| ||||
'                       A7 .. A0 = ++++ ++++ |||| ||||
'                                  |||| |||| |||| ||||
'                                  |||| |||| ++++ ++++ = D7 .. D0
'                                  |||| |||| |||| ||||
'                                  VVVV VVVV VVVV VVVV
PAGEMSK long  %0000_0000_0000_0000_1111_1111_1111_1111
DATAMSK long  %0000_0000_0000_0000_0000_0000_1111_1111
OFFAMSK long  %0000_0000_0000_0000_1111_1111_0000_0000
OFFRMSK long  %0000_1000_0000_0000_1111_1111_0000_0000
OFFWMSK long  %0000_0100_0000_0000_1111_1111_1111_1111
BSEL    long  %0000_1100_0000_0000_1111_1111_1111_1111 
CTRL    long  %0000_1100_0000_0000_0000_0000_0000_0000
'                   ^^
'                   ||
'      page latch = 00
'     !WE (write) = 01
'     !OE  (read) = 10
'          9-VIA  = 11
'
#endif
'----------------- End of MORPHEUS CPU #2 XMM Support Routines -----------------


#elseifdef C3

'------------------------- C3 SPI XMM Support Routines -------------------------
'
' XMM_Activate : Activate the XMM bus. 
'
XMM_Activate
              muxc    XMM_Flags,#1              ' save C flag                                        
              andn    dira,MASK                 ' set I/O pins as inputs while initializing
              or      dira,DIRN                 ' set pin directions                                 
              mov     XMM_Addr,#0               ' set SPI bank 0 ...
              call    #XMM_ModeSelect           ' ... to sequential mode
              mov     XMM_Addr,BIT15            ' set SPI bank 1 ...
              call    #XMM_ModeSelect           ' ... to sequential mode
'
' XMM_UnselectSPIAndReturn - Common return point (all XMM functions jump here!)
'
XMM_UnselectSPIAndReturn
              andn    outa,CS_CLR               ' Reset SPI Channel ...
              or      outa,CS_CLR               ' ... to deselect SPI SRAMs
              shr     XMM_Flags,#1 wc           ' restore C flag
XMM_Activate_ret      
XMM_ReadPage_ret
XMM_ReadLong_ret
XMM_ReadMult_ret
XMM_WritePage_ret
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. Use this to use other hardware that shares
'               pins with the XMM RAM. To reactivate the XMM bus, call XMM_Activate
'            
XMM_Tristate
              andn       outa,CS_CLR            ' Reset SPI Channel ...
              or         outa,CS_CLR            ' ... to deselect SPI SRAMs
              andn       dira,DIRN
XMM_Tristate_ret
              ret

#ifdef NEED_XMM_READLONG              
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 16 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 16 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    Destination register contains bytes read from XMM.
'
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              mov     outx,#%0000_0011          ' Set Command ...
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
              add     XMM_Addr,XMM_Len          ' update XMM_Addr
              mov     XMM_Temp,#0
:ReadLoop
              call    #XMM_InByte               ' read next byte from SPI
              and     outx,#$FF
              or      XMM_Temp,outx
              ror     XMM_Temp,#8
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0,XMM_Temp              ' save the result
              jmp     #XMM_UnselectSPIAndReturn
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 16 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 16 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr incremented by 4.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
XMM_WriteMult
              mov     outx,#%0000_0010          ' Set Command ...               
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
              add     XMM_Addr,XMM_Len          ' update XMM_Addr
XMM_Src       mov     XMM_Temp,0-0              ' save the value to write
:WriteLoop
              mov     outx,XMM_Temp
'              and     outx,#$FF
              call    #XMM_OutByte              ' write a byte to SPI
              ror     XMM_Temp,#8               ' shift next byte down to prepare
              djnz    XMM_Len,#:WriteLoop       ' repeat while XMM_len <> 0
              jmp     #XMM_UnselectSPIAndReturn
'
#endif          
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, 16-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
XMM_WritePage
              mov     outx,#%0000_0010          ' Set Command ...               
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
:WriteLoop
              rdbyte  outx,Hub_Addr             ' get byte from source (Hub)
              call    #XMM_OutByte              ' write byte to SPI
              add     Hub_Addr,#1               ' inc source address
              add     XMM_Addr,#1               ' inc dest address
              djnz    XMM_Len,#:WriteLoop       ' repeat while XMM_len <> 0
              jmp     #XMM_UnselectSPIAndReturn
'
#endif          
'
#ifdef NEED_XMM_READPAGE              
'
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, 16-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
XMM_ReadPage
              mov     outx,#%0000_0011          ' Set Command ...               
              call    #XMM_SetCmdAddr           ' ... and Address (and lower CS)
:ReadLoop
              call    #XMM_InByte               ' read byte from SPI
              wrbyte  outx,Hub_Addr             ' write byte to destination (Hub)
              add     Hub_Addr,#1               ' inc dest address
              add     XMM_Addr,#1               ' inc src address
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
              jmp     #XMM_UnselectSPIAndReturn
'
#endif
'
'
'XMM_SetCmdAddr : Set XMM command and address (and lower CS). XMM_Activate should be called
'                 at least once before this routine is used.
'On Entry:
'   outx     : Command (2 for read, 3 for write)
'   XMM_Addr : address to set (up to 16 bits)
'On Exit:
'   XMM_Addr : preserved
'   C flag saved, then destroyed!
'
XMM_SetCmdAddr
              muxc      XMM_Flags,#1            ' save C flag
              call      #XMM_BankSelect         ' select the SPI SRAM bank
              call      #XMM_OutByte            ' send the command
              mov       XMM_Test,#16            ' 16 address bits
              mov       outx,XMM_Addr           ' put address bits ...
              rol       outx,#16                ' ... in top 16 bits
              call      #XMM_OutBits
XMM_SetCmdAddr_ret
              ret              
'
' XMM_OutByte - send 8 bits to the SPI RAM
' On entry: byte in outx (lower 8 bits)
' On exit: C flag destroyed!
'
' XMM_OutBits - send bits the SPI RAM
' On entry: XMM_Test = number of bits to send ("n")
' On entry: outx contains bits (as top "n" bits)
' On exit: C flag destroyed!
'
XMM_OutByte
              mov       XMM_Test,#8             ' 8 bits
              rol       outx,#24                ' put first bit in MSB
XMM_OutBits              
              rol       outx,#1 wc
              muxc      outa,IBIT
              or        outa,CLCK
              andn      outa,CLCK
              djnz      XMM_Test,#XMM_OutBits
XMM_OutByte_ret
XMM_OutBits_ret
              ret              
'
' XMM_InByte - receive a byte from SPI RAM
' On exit: outx shifted left 8 bits, new byte in lower 8 bits
' On exit: C flag destroyed!
'
XMM_InByte
              mov       XMM_Test,#8             ' 8 bits
:InLoop
              test      OBIT,ina wc
              shl       outx,#1
              muxc      outx,#1
              or        outa,CLCK
              andn      outa,CLCK
              djnz      XMM_Test,#:InLoop      
XMM_InByte_ret
              ret
'
' XMM_BankSelect - select appropriate SPI SRAM bank based on address
' On entry: XMM_Addr bit 15 determines SRAM bank to select (0 or 1)
' On exit:  C flag destroyed!
'
XMM_BankSelect
             andn       outa,CS_CLR             ' Reset SPI Channel ...
             or         outa,CS_CLR             ' ... to deselect SPI SRAMs
             andn       outa,CS_CLK             ' toggle SPI Select incrementor ...
             or         outa,CS_CLK             ' ... to select first SRAM (bank 0)
             test       XMM_Addr,BIT15 wc       ' if bit 15 set ...
       if_c  andn       outa,CS_CLK             ' ... toggle SPI Select incrementor ...
       if_c  or         outa,CS_CLK             ' ... to select second SRAM (bank 1)
XMM_BankSelect_ret
             ret
'
' XMM_ModeSelect - select sequential mode on current bank
' On entry: XMM_Addr bit 15 determines SRAM bank to select (0 or 1)
' On exit:  C flag destroyed!
'
XMM_ModeSelect
              call       #XMM_BankSelect        ' select SPI SRAM bank 0 or 1
              mov        outx,#%0000_0001       ' set ...                                            
              call       #XMM_OutByte           ' ... selected SPI SRAM ...                                   
              mov        outx,#%0100_0000       ' ... to sequential ...                              
              call       #XMM_OutByte           ' ... mode
XMM_ModeSelect_ret
              ret
'
'
' XMM variables
'
outx          long      $0
'
XMM_Flags     long      $0
XMM_Len       long      $0
XMM_Temp      long      $0
XMM_Test      long      $0
XMM_Addr      long      $0
HUB_Addr      long      $0
'
'     
' C3 Hardware Access:
'
MASK          long      (|<Common#SPI_CS_CLK) | (|<Common#SPI_CS_CLR) | (|<Common#SPI_CLK) | (|<Common#SPI_SI) | (|<Common#SPI_SO)
DIRN          long      (|<Common#SPI_CS_CLK) | (|<Common#SPI_CS_CLR) | (|<Common#SPI_CLK) | (|<Common#SPI_SI)       
CS_CLK        long      (|<Common#SPI_CS_CLK)
CS_CLR        long      (|<Common#SPI_CS_CLR)
OBIT          long      (|<Common#SPI_SO)
IBIT          long      (|<Common#SPI_SI)
CLCK          long      (|<Common#SPI_CLK)
BIT15         long      $8000     
'
'---------------------- End of C3 SPI XMM Support Routines ---------------------


#elseifdef TRIBLADEPROP

#ifdef CPU_1

'------------------ TRIBLADEPROP CPU #1 XMM Support Routines -------------------
'
' XMM_Activate : Activate the XMM bus - latches -CE(U23)=0 A19=0 A20=0
'                and makes LE=0 and output. 
'
XMM_Activate
              mov     outa, ram_enable          ' -WE=1 LE=1, -CE = 0 
              mov     dira, ram_dir_read        ' enable bits (for init & wrblock)
              xor     outa, ram_LE_bit          ' LE= 1 -> 0 (latch)
              mov     dira, ram_LE_bit          ' only LE is output
              neg     XMM_Last,#1               ' next call to XMM_SetAddr latches page
XMM_Activate_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. Use this to use other hardware that shares
'               pins with the XMM RAM. To reactivate the XMM bus, call XMM_Activate
'            
XMM_Tristate
              mov     dira, #0                  ' all inputs
              mov     outa, #0                  ' and =0
XMM_Tristate_ret
              ret
'
#ifdef NEED_XMM_READLONG              
'
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 21 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 21 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
' NOTE:  Does not use XMM_IncAddr, but instead increments address directly -
'        this means this routine will only work for longs entirely WITHIN a
'        page - but this should be true for all properly aligned long values.
'        
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              call    #XMM_SetAddr              ' latch ram address
              mov     XMM_Temp, #0              '
:ReadLoop
              mov     outx, ina                 ' read byte from SRAM
              and     outx, #$FF                ' mask the data to 8-bits
              or      XMM_Temp, outx
              ror     XMM_Temp, #8
              add     outa,XMM_Inc              ' inc sram address (no page change)
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0, XMM_Temp             ' save the result
XMM_ReadLong_ret
XMM_ReadMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 21 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr and XMM_Page shifted into position for fast use. XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 21 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page shifted into position for fast use. XMM_Addr incremented by 4.
'
' NOTE:  Does not use XMM_IncAddr, but instead increments address directly -
'        this means this routine will only work for longs entirely WITHIN a
'        page - but this should be true for all properly aligned long values.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
XMM_WriteMult
              call    #XMM_SetAddr              ' latch ram address
XMM_Src       mov     XMM_Temp,0-0              ' save the value
              mov     dira, ram_dir_write       ' set outputs for write
:WriteLoop
              mov     outx,XMM_Temp
              and     outx,#$FF
              or      outx,XMM_Addr
              mov     outa, outx                ' write byte to SRAM
              xor     outa, ram_WE_bit          ' -WE= 1 -> 0  \\ write pulse
              nop                               ' sram delay   ||
              xor     outa, ram_WE_bit          ' -WE= 0 -> 1  //
              ror     XMM_Temp, #8
              add     XMM_Addr,XMM_Inc          ' inc sram address (no page change)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
              mov     dira,ram_dir_read         ' set outputs for read again
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, 19-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
' NOTE:  Uses XMM_IncAddr, which is slower than incrementing the address directly,
'        but it means pages can be correctly read across page boundaries.
'
XMM_WritePage
              call    #XMM_SetAddr              ' latch ram address
              mov     dira, ram_dir_write       ' set outputs for write
:WriteLoop
              rdbyte  outx, Hub_Addr
              or      outx,XMM_Addr
              mov     outa, outx                ' write byte to SRAM
              xor     outa, ram_WE_bit          ' -WE= 1 -> 0  \\ write pulse
              nop                               ' sram delay   ||
              xor     outa, ram_WE_bit          ' -WE= 0 -> 1  //
              add     Hub_Addr, #1              ' inc source address
              call    #XMM_IncAddr              ' inc sram address  (autoinc page)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
              mov     dira,ram_dir_read         ' set outputs for read again
XMM_WritePage_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_READPAGE              
'
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, 19-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
' NOTE:  Uses XMM_IncAddr, which is slower than incrementing the address directly,
'        but it means pages can be correctly read across page boundaries.
'
XMM_ReadPage
              call    #XMM_SetAddr              ' latch ram address
:ReadLoop
              mov     outx, ina                 ' read byte from source (XMM)
              wrbyte  outx, Hub_Addr            ' write byte to destination (Hub)
              add     Hub_Addr, #1              ' inc dest address
              call    #XMM_IncAddr              ' inc sram address (autoinc page)
              mov     outx,XMM_Addr
              or      outx,ram_latched         ' -WE=1 LE=0 + address<<16
              mov     outa,outx                ' setup address incl -WE=1 LE=0
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_ReadPage_ret
              ret
'
#endif              
'
'XMM_SetAddr : Setup initial XMM address. XMM_Activate should be called
'              at least once before this routine is used.
'On Entry:
'   XMM_Addr : address to set (19 bits)
'On Exit:
'   XMM_Addr : 19 bits of address, shifted into position for fast use.
'   XMM_Last : 8 bits of page, shifted into position for fast use
'
XMM_SetAddr
              or      dira,ram_dir_read         ' set outputs for read
              mov     XMM_Page,XMM_Addr         ' shift  ...
              shl     XMM_Page,#5               ' ... ram page ...
              and     XMM_Page,ram_page_shft    ' ... into position for latching
              mov     XMM_Test,XMM_Page         ' same page ...
              sub     XMM_Test,XMM_Last         ' ... as last page ?
              tjz     XMM_Test,#XMM_OutAddr     ' yes - just output the address
              mov     XMM_Last,XMM_Page         ' no - save new page
              mov     outx,ram_enable           ' -WE=1 LE=1, -CE = 0
              or      outx,XMM_Page             ' include page 
              mov     outa,outx                 ' -WE=1 LE=1, -CE = 0
              xor     outa,ram_LE_bit           ' LE= 1 -> 0 (latch)
XMM_OutAddr
              shl     XMM_Addr,#16              ' extract address in page ...
              and     XMM_Addr,ram_addr_shft    ' ... and shift into position
              or      XMM_Addr,ram_latched      ' -WE=1 LE=0 + address<<16
              mov     outx,XMM_Addr
              mov     outa,outx                 ' setup address incl -WE=1 LE=0
XMM_SetAddr_ret
              ret
'
#ifdef NEED_XMM_INCADDR              
'
'
'XMM_IncAddr : Increment XMM address, changing page if necessary. XMM_SetAddr
'              must be called at least once before this routine is used, to
'              separate address bits from page/chip bits, and shift them into
'              the correct position for use.
'On Entry:
'   XMM_Addr : 11 bits of address, shifted into position for fast use.
'   XMM_Last : 8 bits of page, shifted into position for fast use.
'On Exit:
'   XMM_Addr : incremented (wraps within page)
'   XMM_Last : incremented if the page number changes (and latch updated).
'
' NOTES: While slower than incrementing the memory address directly, this
'        routine automatically copes with XMM page changes.
'
XMM_IncAddr
              add     XMM_Addr,XMM_Inc          ' inc shifted sram address
              mov     XMM_Test,XMM_Addr         ' is result of incrementing ...
              and     XMM_Test,ram_addr_shft    ' ... address zero (i.e. on next page)?
              tjnz    XMM_Test,#XMM_IncAddr_ret ' no - done
              add     XMM_Last,XMM_Inc          ' yes - increment ...
              and     XMM_Last,ram_page_shft    ' ... page instead 
              shr     XMM_Last,#5               ' rebuild ...
              mov     XMM_Addr,XMM_Last         ' ... full address ...
              neg     XMM_Last,#1               ' ... and force latching of new page
              call    #XMM_SetAddr              ' set new address and page 
XMM_IncAddr_ret
              ret
'
#endif
'
' TriBladeProp CPU #1 Hardware Access:
'
'               MOUSE ---------+         +-------- VGA/TV                                      
'               MOUSE --------+|         |+------- VGA/TV                  
'                  KB -------+||         ||+------ VGA/TV                   
'     -CE/-CS ---- KB -----+ |||         |||+----- VGA/TV                   
'  LE/CLK/SCL ------------+| |||         ||||+---- VGA/MOUSE    
'     -WE/SDA -----------+|| |||         |||||+--- VGA/MOUSE                                    
'    (P30) SO ----------+||| |||         ||||||+-- VGA/KB                                        
'    (P31) SI ---------+|||| |||         |||||||+- VGA/KB                                     
'                      ||||| |||         ||||||||                                                          
ram_WE_bit    long    %00100_00000000000_00000000_00000000 ' WE bit
ram_LE_bit    long    %00010_00000000000_00000000_00000000 ' LE bit
ram_latched   long    %00100_00000000000_00000000_00000000 ' -WE=1 LE=0
ram_enable    long    %00110_00000000000_00000000_00000000 ' -WE=1 LE=1 + -CE = 0
ram_dir_read  long    %00111_11111111111_00000000_00000000 ' outputs = WE, LE, A0-18, inputs D0-7
ram_dir_write long    %00111_11111111111_00000000_11111111 ' outputs = WE, LE, A0-18, D0-7
ram_addr_shft long    %00000_11111111111_00000000_00000000 ' address in 2k page (shifted into position)
ram_page_shft long    %00000_00011111111_00000000_00000000 ' 2k page (shifted into position)
'                            A10......A0          D7....D0
'                               Q7....Q0
'                               |      |
'                               +------+
'                                  ||
'                             Latched pins
'                             (A18 .. A11)         
'                            passes when LE=1                                            
'                            latches on  LE low going edge                              -
'                            latched on  LE=0             
'
outx          long      0                       ' as a temp address + WE/OE/LE bits
'
XMM_Addr      long      0
XMM_Page      long      0                       ' current page
XMM_Last      long      -1                      ' last page - initially set to impossible value
XMM_Temp      long      0
XMM_Len       long      0
XMM_Inc       long      1 << 16                 ' 1 shifted for incrementing addr/page
XMM_Test      long      0
HUB_Addr      long      0 
'
'--------------- End of TRIBLADEPROP CPU #1 XMM Support Routines --------------
'
#elseifdef CPU_3
'
  ERROR : TRIBLADEPROP HAS NO XMM RAM ON CPU_3
'
#else
'
'------------------ TRIBLADEPROP CPU #2 XMM Support Routines ------------------
'
' XMM_Activate : activate the XMM bus - latches -CE(U23)=0 A19=0 A20=0
'                and makes LE=0 and output. 
'
XMM_Activate
              mov     outa, ram_enable_lo       ' put latch back to SRAM=U23
              mov     dira, ram_dir_read        ' enable bits (for init & wrblock)
              xor     outa, ram_LE_bit          ' LE= 1 -> 0 (latch)
              mov     dira, ram_LE_bit          ' only LE is output
              neg     XMM_Last,#1               ' next call to XMM_SetAddr latches page
XMM_Activate_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. Call this to allow other cogs to use microSD etc.
'               To activate XMM again, call XMM_Activate. Note that to avoid resetting
'               the other CPUs, and also so that the other cogs do not corrupt the
'               XMM RAM, we leave RST1,RST3 and the RAM chip selects high - this means
'               the other cogs cannot use XMM RAM or reset the other CPUs - if this
'               is required then this function may need to be modified - but carefully! 
'
XMM_Tristate
              mov     dira,ram_tristate         ' make sure RST1,RST3,CS(U23) and CS(U24) ...
              mov     outa,ram_tristate         ' ... remain output and high - others are inputs
XMM_Tristate_ret
              ret
'
#ifdef NEED_XMM_READLONG              
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 22 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 22 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
' NOTE:  Does not use XMM_IncAddr, but instead increments address directly -
'        this means this routine will only work for longs entirely WITHIN a
'        page - but this should be true for all properly aligned long values.
'        
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              call    #XMM_SetAddr              ' latch ram address
              mov     XMM_Temp, #0              '
              xor     outa, ram_OE_bit          ' -OE = 1 -> 0
:ReadLoop
              mov     outx, ina                 ' read byte from SRAM
              and     outx, #$FF                ' mask the data to 8-bits
              or      XMM_Temp, outx
              ror     XMM_Temp, #8
              add     outa,XMM_Inc              ' inc sram address (no page change)
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0, XMM_Temp             ' save the result
XMM_ReadLong_ret
XMM_ReadMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 22 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr and XMM_Page shifted into position for fast use. XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 22 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page shifted into position for fast use. XMM_Addr incremented by 4.
'
' NOTE:  Does not use XMM_IncAddr, but instead increments address directly -
'        this means this routine will only work for longs entirely WITHIN a
'        page - but this should be true for all properly aligned long values.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
XMM_WriteMult
              call    #XMM_SetAddr              ' latch ram address
XMM_Src       mov     XMM_Temp,0-0              ' save the value
              mov     dira, ram_dir_write       ' set outputs for write
:WriteLoop
              mov     outx,XMM_Temp
              and     outx,#$FF
              or      outx,XMM_Addr
              mov     outa, outx                ' write byte to SRAM
              xor     outa, ram_WE_bit          ' -WE= 1 -> 0  \\ write pulse
              nop                               ' sram delay   ||
              xor     outa, ram_WE_bit          ' -WE= 0 -> 1  //
              ror     XMM_Temp, #8
              add     XMM_Addr,XMM_Inc          ' inc sram address (no page change)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
              mov     dira,ram_dir_read         ' set outputs for read again
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, 22-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
' NOTE:  Uses XMM_IncAddr, which is slower than incrementing the address directly,
'        but it means bytes can be correctly read across SRAM page boundaries.
'
XMM_WritePage
              call    #XMM_SetAddr              ' latch ram address
              mov     dira, ram_dir_write       ' set outputs for write
:WriteLoop
              rdbyte  outx, Hub_Addr
              or      outx,XMM_Addr
              mov     outa, outx                ' write byte to SRAM
              xor     outa, ram_WE_bit          ' -WE= 1 -> 0  \\ write pulse
              nop                               ' sram delay   ||
              xor     outa, ram_WE_bit          ' -WE= 0 -> 1  //
              add     Hub_Addr, #1              ' inc source address
              call    #XMM_IncAddr              ' inc sram address  (autoinc page)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
              mov     dira,ram_dir_read         ' set outputs for read again
XMM_WritePage_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_READPAGE              
'
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, 19-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
' NOTE:  Uses XMM_IncAddr, which is slower than incrementing the address directly,
'        but it means bytes can be correctly read across SRAM page boundaries.
'
XMM_ReadPage
              call    #XMM_SetAddr              ' latch ram address
              xor     outa, ram_OE_bit          ' -OE = 1 -> 0
:ReadLoop
              mov     outx, ina                 ' read byte from source (XMM)
              wrbyte  outx, Hub_Addr            ' write byte to destination (Hub)
              add     Hub_Addr, #1              ' inc dest address
              call    #XMM_IncAddr              ' inc sram address (autoinc page)
              mov     outa, XMM_Addr            ' setup address incl -WE=1 -OE=1 LE=0
              xor     outa, ram_OE_bit          ' -OE = 1 -> 0
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_ReadPage_ret
              ret
'
#endif
'
'XMM_SetAddr : Setup initial XMM address. XMM_Activate should be called
'              at least once before this routine is used.
'On Entry:
'   XMM_Addr : address to set (up to 22 bits)
'On Exit:
'   XMM_Addr : 19 bits of ram address bits, shifted into position for use.
'   XMM_Last : up to 3 bits of page/chip select bits, shifted into position for use.
'
XMM_SetAddr
              mov     dira, ram_dir_read        ' set outputs
              mov     XMM_Page,XMM_Addr         ' shift ...
              shr     XMM_Page,#11              ' ... SRAM page/chip bits ...
              and     XMM_Page,ram_pacs_shft    ' ... into position
              mov     XMM_Test,XMM_Page         ' same SRAM page/chip ...
              sub     XMM_Test,XMM_Last         ' ... as last used?
              tjz     XMM_Test,#XMM_OutAddr     ' yes - just output the address
              muxnz   XMM_Test,#1               ' no - save Z flag
              test    XMM_Page,ram_chip_shft wz ' test SRAM chip select bit
       if_z   mov     outx, ram_enable_lo       ' -WE=1 -OE=1 LE=1, -CE(U23 SRAM) = 0 (A19-20=0)
       if_nz  mov     outx, ram_enable_hi       ' -WE=1 -OE=1 LE=1, -CE(U24 SRAM) = 0 (A19-20=0)
              cmp     XMM_Test,#0 wz            ' restore Z flag
              mov     XMM_Last,XMM_Page         ' save new SRAM page/chip bits 
              and     XMM_Page,ram_page_shft    ' use SRAM page bits ...
              or      outx, XMM_Page            ' ... for latching
              mov     outa, outx                ' -WE=1 -OE=1 LE=1, -CE(U23)=0 or -CE(U24)= 0 (A19-20= page)
              xor     outa, ram_LE_bit          ' LE= 1 -> 0 (latch)
XMM_OutAddr
              shl     XMM_Addr,#8               ' shift address bits into position ...
              and     XMM_Addr,ram_addr_shft    ' ... and mask off page/chip bits
              or      XMM_Addr, ram_latched     ' -WE=1 -OE=1 LE=0 + address<<8
              mov     outa, XMM_Addr            ' setup address incl -WE=1 -OE=1 LE=0
XMM_SetAddr_ret
              ret
'
#ifdef NEED_XMM_INCADDR              
'
'
'XMM_IncAddr : Increment XMM address, changing page if necessary. XMM_SetAddr
'              must be called at least once before this routine is used, to
'              separate address bits from page/chip bits, and shift them into
'              the correct position for use.
'On Entry:
'   XMM_Addr : 22 bits of address, shifted into position for fast use.
'   XMM_Last : up to 3 bits of page/chip select, shifted into position for fast use
'On Exit:
'   XMM_Addr : incremented (wraps within page)
'   XMM_Page : incremented if the page number changes (and latch updated).
'
' NOTES: While slower than incrementing the memory address directly, this
'        routine automatically copes with XMM page changes.
'
XMM_IncAddr
              add     XMM_Addr,XMM_Inc          ' inc shifted sram address
              mov     XMM_Test,XMM_Addr         ' is result of incrementing ...
              and     XMM_Test,ram_addr_shft    ' ... address zero (i.e. on next page)?
              tjnz    XMM_Test,#XMM_IncAddr_ret ' no - done
              add     XMM_Last,XMM_Inc          ' yes - increment ...
              and     XMM_Last,ram_page_shft    ' ... page instead 
              shl     XMM_Last,#11              ' rebuilt ...
              mov     XMM_Addr,XMM_Last         ' ... full address ...
              neg     XMM_Last,#1               ' ... and force latching of new page
              call    #XMM_SetAddr              ' set new address and page 
XMM_IncAddr_ret
              ret
'
#endif              
'
' TriBladeProp CPU #2 Hardware Access:
'
'                                        +-------- -CS (U26 FLASH)   -+                                    
'                                        |+------- -CS (J22 microSD)  +- Latched pins...                   
'                                        ||+------ -RST3              |    passes when LE=1                
'           (P30) SO ---+                |||+----- -RST1              |    latches on  LE low going edge   
'           (P31) SI --+|                ||||+---- -CE (U24 SRAM-hi)  |    latched on  LE=0                
'                      ||+---- -WE(SDA)  |||||+--- -CE (U23 SRAM-lo)  |                                    
'                      |||+--- -OE(SCL)  ||||||+-- A20                |                                    
'                      ||||+--  LE       |||||||+- A19               -+                                    
'                      |||||             ||||||||                                                          
ram_WE_bit    long    %00100_00000000000_00000000_00000000 ' WE bit
ram_OE_bit    long    %00010_00000000000_00000000_00000000 ' OE bit
ram_LE_bit    long    %00001_00000000000_00000000_00000000 ' LE bit
ram_latched   long    %00110_00000000000_00000000_00000000 ' -WE=1 -OE=1 LE=0                                 
ram_enable_lo long    %00111_00000000000_11111000_00000000 ' -WE=1 -OE=1 LE=1 + -CE(U23 SRAM) = 0 (A19-20=0)
ram_enable_hi long    %00111_00000000000_11110100_00000000 ' -WE=1 -OE=1 LE=1 + -CE(U24 SRAM) = 0 (A19-20=0)
ram_dir_read  long    %00111_11111111111_11111111_00000000 ' outputs = WE, OE, LE, A0-18, inputs D0-7
ram_dir_write long    %00111_11111111111_11111111_11111111 ' outputs = WE, OE, LE, A0-18, D0-7
ram_addr_shft long    %00000_11111111111_11111111_00000000 ' SRAM address bits (any SRAM size), in position
ram_pacs_shft long    %00000_00000000000_00000111_00000000 ' SRAM page/chip bits (any SRAM size), in position
ram_page_shft long    %00000_00000000000_00000011_00000000 ' SRAM page bits only (any SRAM size), in position
ram_tristate  long    %00000_00000000000_00111100_00000000 ' -RST1=1, -RST3=1, -CE(U24)=1, -CE(U23)=1
'                            +---------+ +------+ +------+
'                            A18......A8 A7....A0 D7....D0
'                                        Q7....Q0
'
' The following value depends on the type of RAM chips installed:
'
ram_chip_shft long    %00000_00000000000_00000001_00000000 ' SRAM chip select bit (for 512Kb SRAM), in position 
'
outx          long      0                       ' as a temp address + WE/OE/LE bits
'
XMM_Addr      long      0
XMM_Page      long      0                       ' current page
XMM_Last      long      -1                      ' last page - initially set to impossible value
XMM_Temp      long      0
XMM_Len       long      0
XMM_Inc       long      1 << 8                  ' 1 shifted for incrementing addr/page
XMM_Test      long      0
HUB_Addr      long      0
'
'----------------- End of TRIBLADEPROP CPU #2 XMM Support Routines ------------

#endif

#elseifdef RAMBLADE

'------------------------ RAMBLADE XMM Support Routines -----------------------
'
' XMM_Activate : activate the XMM bus - set pin directions and CE for read
'
XMM_Activate
              mov     outa, #0                  ' zero all outputs
              mov     dira, ram_addr_only       ' set address bits as outputs
XMM_Activate_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. Call this to allow other cogs to use other devices.
'
XMM_Tristate
              mov     outa, #0                  ' zero all outputs
              mov     dira, #0                  ' disable all outputs
XMM_Tristate_ret
              ret
'
#ifdef NEED_XMM_READLONG              
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 19 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 19 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              and     XMM_Addr,ram_addr_only    ' mask address to 19 bits
              mov     outa, XMM_Addr            ' output the address (-CE =0 => read)
              mov     XMM_Temp, #0              '
:ReadLoop
              mov     dira,ram_dir_read
              nop                               ' REQUIRED for timing
              mov     outx, ina                 ' read byte from SRAM
              mov     dira,ram_addr_only
              shr     outx, #24                 ' move bits into position
              or      XMM_Temp, outx
              ror     XMM_Temp, #8
              add     outa, #1                  ' inc sram address
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0, XMM_Temp             ' save the result
XMM_ReadLong_ret
XMM_ReadMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 19 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 19 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr incremented by 4.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
              and     XMM_Addr,ram_addr_only    ' mask address to 19 bits
              mov     dira,ram_addr_data        ' set bits for write (but -CE -WE =0 => deselect)
XMM_WriteMult
XMM_Src       mov     XMM_Temp,0-0              ' save the value
:WriteLoop
              mov     outx,XMM_Temp             ' set up ...
              shl     outx, #24                 ' ... address ...
              or      outx,XMM_Addr             ' ... and ...
              mov     outa,outx                 ' ... data bits
              mov     dira,ram_dir_write        ' (-CE -WE =1 => write)
              ror     XMM_Temp, #8
              add     Hub_Addr, #1              ' inc source address
              add     XMM_Addr, #1              ' inc XMM address
              mov     dira,ram_addr_only        ' set outputs for address (-CE -WE =0 => deselect)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, 22-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
XMM_WritePage
              and     XMM_Addr,ram_addr_only    ' mask address to 19 bits
              mov     dira,ram_addr_data        ' set bits for write (but -CE -WE =0 => deselect)
:WriteLoop
              rdbyte  outx, Hub_Addr            ' set up ...
              shl     outx, #24                 ' ... address ...
              or      outx,XMM_Addr             ' ... and ...
              mov     outa,outx                 ' ... data bits
              mov     dira,ram_dir_write        ' (-CE -WE =1 => write)
              add     Hub_Addr, #1              ' inc source address
              add     XMM_Addr, #1              ' inc XMM address
              mov     dira,ram_addr_only        ' (-CE -WE =0 => deselect)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
XMM_WritePage_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_READPAGE              
'
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, 19-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
XMM_ReadPage
              and     XMM_Addr,ram_addr_only    ' mask address to 19 bits
              mov     outa, XMM_Addr            ' output the address (-CE =0 => read)
:ReadLoop
              mov     dira,ram_dir_read
              nop                               ' REQUIRED for timing
              mov     outx, ina                 ' read byte from SRAM
              mov     dira,ram_addr_only
              shr     outx,#24                  ' shift data bits into position
              wrbyte  outx, Hub_Addr            ' write byte to destination (Hub)
              add     outa,#1                   ' inc sram address 
              add     Hub_Addr, #1              ' inc dest address
              djnz    XMM_Len, #:ReadLoop       ' repeat while XMM_len <> 0
XMM_ReadPage_ret
              ret
'
#endif
'
'XMM_SetAddr : Setup initial XMM address. XMM_Activate should be called
'              at least once before this routine is used.
'On Entry:
'   XMM_Addr : address to set (up to 19 bits)
'On Exit:
'   XMM_Addr : 19 bits of ram address.
'
'XMM_IncAddr : Increment XMM address, changing page if necessary. 
'On Entry:
'   XMM_Addr : 19 bits of ram address.
'On Exit:
'   XMM_Addr : incremented 
'
'#ifdef NEED_XMM_INCADDR              
'XMM_IncAddr   add     XMM,Addr,#1               ' increment address
'#endif              
'XMM_SetAddr
'              and     XMM_Addr,ram_addr_bits    ' mask address to 19 bits
'              mov     outa, XMM_Addr            ' output the address (& -CE=0)
'              mov     dira, ram_addr_bits       ' set the adress outputs
'              or      dira, ram_CE_bit          ' set CE output (-CE=0 => Read)
'XMM_SetAddr_ret
'XMM_IncAddr_ret
'              ret
'
'
' RamBlade Hardware Access:
'
'       P0-18  = A0-18   SRAM address
'       P19    = -CS     microSD
'       P20    = -CE     SRAM & enable eeprom
'       P21    = -WE     SRAM & enable eeprom
'       P22    = SO      \ not used by this driver
'       P23    = SI      /
'       P24-31 = D0-7    SRAM \ * see below
'                             | P28 = SCL eeprom
'                             | P29 = SDA eeprom
'                             | P30 = prog SO
'                             / P31 = prog SI
'
'         (microSD) DO ---------+
'         (microSD) DI --------+| +--------  SI (P23)
'         (microSD) CLK-------+|| |+-------  SO (P22)
'                             ||| ||
'             (P28) SCL ----+ ||| ||+------ -WE (SRAM & enable eeprom)
'             (P29) SDA ---+| ||| |||+----- -CE (SRAM & enable eeprom)
'             (P30) SO ---+|| ||| ||||+---- -CS (microSD)
'             (P31) SI --+||| ||| |||||
'                        |||| ||| |||||
'          P31...P0 --> %xxxxxxxx_00xxx_xxxxxxxxxxxxxxxxxxx
'       10K pullups -->    ** ***   ***
'                        +------+       +-----------------+
'                        D7....D0       A18..............A0
ram_CE_bit    long      %00000000_00010_0000000000000000000 ' CE bit
ram_WE_bit    long      %00000000_00100_0000000000000000000 ' WE bit
ram_dir_read  long      %00000000_00010_1111111111111111111 ' outputs CE, A0-18, inputs WE(t/s),D0-7
ram_dir_write long      %11111111_00110_1111111111111111111 ' outputs WE, CE, A0-18, D0-7
ram_addr_only long      %00000000_00000_1111111111111111111 ' address bits A0-18
ram_addr_data long      %11111111_00000_1111111111111111111 ' address bits A0-18, data bits D0-7
'
'
'
outx          long      0
'
XMM_Addr      long      0
XMM_Temp      long      0
XMM_Len       long      0
HUB_Addr      long      0
'
'----------------------- End of RAMBLADE XMM Support Routines -------------------

 
#elseifdef DRACBLADE
'
'------------------------- DRACBLADE XMM Support Routines -----------------------
'
' XMM_Activate : Activate the XMM bus - set Pin directions, and latch for Read
'
XMM_Activate
              mov     outa,ram_MUX              ' disable mux (latch is active low)
              mov     dira,ram_dir_read         ' set direction bits for read
XMM_Activate_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. To reactivate, call XMM_Activate
'            
XMM_Tristate
              mov     dira,#0                   ' all inputs
              mov     outa,#0                   ' and all zero
XMM_Tristate_ret
              ret
'
#ifdef NEED_XMM_READLONG              
'
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 21 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 21 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
' NOTE:  Does not use XMM_IncAddr, but instead increments address directly -
'        this means this routine will only work for longs entirely WITHIN a
'        page - but this should be true for all properly aligned long values.
'        
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              call    #XMM_SetAddr              ' latch ram address
              mov     XMM_Temp, #0
:ReadLoop
              andn    outa,ram_MUX              ' enable mux
              nop                               ' sram delay              
              nop                               ' sram delay              
              mov     outx,ina                  ' read byte from sram
              or      outa,ram_MUX              ' disable mux
              and     outx,#$FF                 ' mask the data to 8-bits
              or      XMM_Temp,outx             ' ... into ...
              ror     XMM_Temp,#8               ' ... XMM_Temp
              call    #XMM_IncAddr              ' inc sram address
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0, XMM_Temp             ' save the result
XMM_ReadLong_ret
XMM_ReadMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 21 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr and XMM_Page shifted into position for fast use. XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 21 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page shifted into position for fast use. XMM_Addr incremented by 4.
'
' NOTE:  Does not use XMM_IncAddr, but instead increments address directly -
'        this means this routine will only work for longs entirely WITHIN a
'        page - but this should be true for all properly aligned long values.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
XMM_WriteMult
              call    #XMM_SetAddr              ' latch ram address
XMM_Src       mov     XMM_Temp,0-0              ' save the value
:WriteLoop
              mov     dira, ram_dir_write       ' set outputs for write (to sram)
              mov     outx,XMM_Temp             ' get next ...
              and     outx,#$FF                 ' .. byte to write
              or      outx,ram_WR               ' set write and disable mux 
              mov     outa,outx                 ' write to SRAM
              andn    outa,ram_MUX              ' enable mux
'             nop                               ' sram delay
              or      outa,ram_MUX              ' disable mux
              ror     XMM_Temp,#8               ' prepare to write next byte
              call    #XMM_IncAddr              ' inc sram address (autoinc page)
              djnz    XMM_Len,#:WriteLoop       ' repeat while XMM_len <> 0
              mov     dira,ram_dir_read         ' set outputs for read again
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, 19-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
' NOTE:  Uses XMM_IncAddr, which is slower than incrementing the address directly,
'        but it means pages can be correctly read across page boundaries.
'
XMM_WritePage
              call    #XMM_SetAddr              ' latch ram address
:WriteLoop
              mov     dira,ram_dir_write        ' set outputs for write (to sram)
              rdbyte  outx,Hub_Addr             ' get byte to write
              or      outx,ram_WR               ' set write and disable mux 
              mov     outa,outx                 ' write byte (to sram)
              andn    outa,ram_MUX              ' enable mux
'             nop                               ' sram delay
              or      outa,ram_MUX              ' disable mux
              add     Hub_Addr,#1               ' inc source address
              call    #XMM_IncAddr              ' inc sram address (autoinc page)
              djnz    XMM_Len,#:WriteLoop       ' repeat while XMM_len <> 0
              mov     dira,ram_dir_read         ' set outputs for read again
XMM_WritePage_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_READPAGE              
'
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, 19-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
' NOTE:  Uses XMM_IncAddr, which is slower than incrementing the address directly,
'        but it means pages can be correctly read across page boundaries.
'
XMM_ReadPage
              call    #XMM_SetAddr              ' latch ram address
:ReadLoop
              andn    outa,ram_MUX              ' enable mux
              nop                               ' sram delay              
              nop                               ' sram delay              
              mov     outx,ina                  ' read byte (from sram)
              or      outa,ram_MUX              ' disable mux
              wrbyte  outx,Hub_Addr             ' write byte to destination (Hub)
              add     Hub_Addr,#1               ' inc dest address
              call    #XMM_IncAddr              ' inc sram address (autoinc page)
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
XMM_ReadPage_ret
              ret
'
#endif              
'
'XMM_SetAddr : Setup initial XMM address. XMM_Activate should be called
'              at least once before this routine is used.
'On Entry:
'   XMM_Addr      : address to set (19 bits)
'   XMM_Last0_7   : Last address bits 0 to 7 (or -1 if no last addr) 
'   XMM_Last8_15  : Last address bits 8 to 15 (in bits 0 .. 7, or -1)
'   XMM_Last16_19 : LAst address bits 16 to 19 (in bits 0 .. 7, or -1)
'
'On Exit:
'   latches set
'   XMM_Last0_7, XMM_Last8_15, XMM_Last_16_19 : updated
'
XMM_SetAddr
              mov     dira,ram_dir_write        ' set up for writing (to latch)
              mov     outx,XMM_Addr             ' shift  ...
              shr     outx,#16                  ' addr bits 16 - 19 ...
              and     outx,#$FF                 ' ... into position
              or      outx,ram_A16_A19          ' set mux for bits 16 - 19
              mov     outa,outx                 ' output new bits and enable mux
              andn    outa,ram_MUX              ' enable mux
'             nop                               ' sram delay              
              or      outa,ram_MUX              ' disable mux
XMM_Check8_19
              mov     outx,XMM_Addr             ' shift  ...
              shr     outx,#8                   ' addr bits 8 - 15 ...
              and     outx,#$FF                 ' ... into position
              or      outx,ram_A8_A15           ' set mux for bits 8 - 15
              mov     outa,outx                 ' output new bits and enable mux
              andn    outa,ram_MUX              ' enable mux
'             nop                               ' sram delay              
              or      outa,ram_MUX              ' disable mux
XMM_Check0_7
              mov     outx,XMM_Addr             ' bits 0 - 7 ...
              and     outx,#$FF                 ' ... are already in position
              or      outx,ram_A0_A7            ' set mux for bits 0 - 7 
              mov     outa,outx                 ' output new bits and enable mux
              andn    outa,ram_MUX              ' enable mux
'             nop                               ' sram delay              
              or      outa,ram_MUX              ' set read and disable mux
XMM_Check_Done
              mov     outa,ram_MUX
              mov     dira,ram_dir_read         ' set up for reading
XMM_SetAddr_ret
              ret
'
'#ifdef NEED_XMM_INCADDR
' DRACBLADE ALWAYSE NEEDS INCADDR              
'
'
'XMM_IncAddr : Increment XMM address, setting latches as necessary. XMM_Activate
'              must be called at least once before this routine is used.
' On Entry:
'   XMM_Addr      : address to increment (19 bits)
'   XMM_Last0_7   : Last address bits 0 to 7 (or -1 if no last addr) 
'   XMM_Last8_15  : Last address bits 8 to 15 (in bits 0 .. 7, or -1)
'   XMM_Last16_19 : LAst address bits 16 to 19 (in bits 0 .. 7, or -1)
'
'On Exit:
'   latches set
'   XMM_Last0_7, XMM_Last8_15, XMM_Last_16_19 : updated
'
XMM_IncAddr
              add     XMM_Addr,#1               ' inc sram address
              mov     outx,XMM_Addr             ' does result of incrementing ...
              and     outx,#$FF                 ' ... require updating latch 8 - 15 or 16 - 19?
              tjnz    outx,#XMM_Set0_7          ' if not, just set latch for addr bits 0 - 7
              call    #XMM_SetAddr              ' otherwise we must set all latches
              jmp     #XMM_IncAddr_ret          ' done 
XMM_Set0_7
              mov     dira,ram_dir_write        ' set up for writing (to latch)
              or      outx,ram_A0_A7            ' set mux for bits 0 - 7 
              mov     outa,outx                 ' output new bits and enable mux
              andn    outa,ram_MUX              ' enable mux
'             nop                               ' sram delay              
              or      outa,ram_MUX              ' set read and disable mux
              mov     outa,ram_MUX
              mov     dira,ram_dir_read         ' set up for reading
XMM_IncAddr_ret
              ret
'
'#endif
'
' DracBlade Hardware Access:
'
'   (P24) M_PORT2-------------+ +-------- VGA (P23)                                             
'   (P25) M_PORT2------------+| |+------- VGA (P22)                          
'   (P26) KB ---------------+|| ||+------ VGA (P21)                           
'   (P27) KB --------------+||| |||+----- VGA (P20)                        
'   (P28) SCL ------------+|||| ||||+---- VGA (P19)            
'   (P29) SDA -----------+||||| |||||+--- VGA (P18)                                               
'   (P30) SO -----------+|||||| ||||||+-- VGA (P17)                                                
'   (P31) SI ----------+||||||| |||||||+- VGA (P16)                                                      
'                      |||||||| ||||||||                                                       
ram_MUX       long    %00000000_00000000_0000_0001_00000000 ' active low                                        
'ram_RD        long    %00000000_00000000_0000_0001_00000000 ' same as ram_MUX
ram_WR        long    %00000000_00000000_0000_0011_00000000
ram_A0_A7     long    %00000000_00000000_0000_0101_00000000
ram_A8_A15    long    %00000000_00000000_0000_0111_00000000
ram_A16_A19   long    %00000000_00000000_0000_1001_00000000
'ram_DATA      long    %00000000_00000000_0000_0000_11111111
ram_dir_read  long    %00000000_00000000_0000_1111_00000000
ram_dir_write long    %00000000_00000000_0000_1111_11111111
'                                        |||| |||| ||||||||
'                         SD CS ---(P15)-+||| |||| |-DATA-|
'                         SD DI ---(P14)--+|| |||| 76543210
'                         SD CLK --(P13)---+| ||||
'                         SD DO ---(P12)----+ |||+--(P8) ---- /mux (active low) 
'                                             ||+---(P9)---+                               
'                                             |+---(P10)--+|                   
'                                             +----(P11)-+|| 
'                                                        |||               
'                                                        |||
'                                                        000 /RD_RAM
'                                                        001 /WR_RAM
'                                                        010 /CS_A0-A7
'                                                        011 /CS_A8-A15
'                                                        100 /CS_A16-A18
'                                                        101 /CS_LCD
'                                                        110 Not connected
'                                                        111 Not connected
'
outx          long      0                       ' as a temp address
'
XMM_Addr      long      0
XMM_Temp      long      0
XMM_Len       long      0
XMM_Test      long      0
HUB_Addr      long      0
'
'----------------------- End of DRACBLADE XMM Support Routines ------------------

#else

'-------------------------- HX512 XMM Support Routines -------------------------
'
' XMM_Activate : Activate the XMM bus. Setup XMM card for auto increment on read & write.
'
XMM_Activate
              ' set the IO direction and states for SRAM interface
              ' write 1's to interface, control lines only. so we don't accidentally fire a program clock
              mov     outa,#%0000_0110 ' HX_CTRL_MASK
              ' one by one set pin groups to output
              or      dira,HX_OUTPUT_MASK       ' set I/Os for sram interface all to outputs for now
              andn    outa,#%0000_0110 ' HX_CTRL_MASK         ' set control lines to code "00" which means next clock strobe read program
#ifdef ACTIVATE_INITS_XMM
'
' NOTE: This code must be disabled in the Kernel, since the Kernel can never actually initialize the HX512.
'       This is a limitation of the design of the HX512, which can be initialized only once after a reset.
'       Since the program must have been loaded into XMM prior to the Kernel executing, the HX512 must have
'       already been initialized - and attempting to initialize it again can give erroneous results.

              ' initialize the memory controller, need to put the command on the data bus, and pulse strobe
              andn    outa,HX_DBUS_MASK         ' outa = (outa & !HX_DBUS_MASK), make hole for data
              mov     outx,#%0000_1111          ' outx = program code for sram controller
              shl     outx,#HX_DBUS_BIT_SHIFT   ' outx < 16, place data into proper position
              or      outa,outx                 ' finally, outa = (outa & !HX_DBUS_MASK) | outx
              ' now pulse the strobe line
              or      outa,HX_STROBE_MASK       ' strobe = 1
              andn    outa,HX_STROBE_MASK       ' strobe = 0
#endif              
              ' now that we are done, set DBUS to inputs, leave control to outputs.
              andn    outa,HX_OUTPUT_MASK 
              andn    dira,HX_DBUS_MASK
              neg     XMM_Last,#1               ' force call to set XMM_SetAddr
XMM_Activate_ret
              ret
'
'XMM_Tristate : Give up the XMM bus. Call this to allow other cogs to use microSD etc.
'               To activate XMM again, call XMM_Activate.  
'
XMM_Tristate
              nop                               ' TBD
XMM_Tristate_ret
              ret
'
#ifdef NEED_XMM_READLONG              
'
' XMM_ReadLong : Read long from XMM at address in XMM_Addr into the destination register
'                set up in XMM_Dest. XMM_Activate should be called at least once before
'                this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 22 bits)
'    XMM_Dst  : destination of this instruction set to destination register
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
'
' XMM_ReadMult : Read multiple bytes in source register to XMM at address XMM_Addr.
'                XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to read (up to 22 bits)
'    XMM_Dst  : destination of this instruction set to destination register
'    XMM_Len  : number of bytes to read (usually 1, 2 or 4)
' On exit:
'    XMM_Addr and XMM_Page set up for fast access. XMM_Addr incremented by 4.
'    Destination register contains long read from XMM.
'
XMM_ReadLong
              mov     XMM_Len,#4                ' read 4 bytes
XMM_ReadMult
              call    #XMM_TestAndSetAddr       ' Test address and set if required
              mov     XMM_Temp, #0              '
:XMM_Last_ok
              andn    dira,HX_DBUS_MASK         ' place data bus into read mode  
              ' command lines should already be in output mode,
              ' so only need to write 2-bit command code for read memory
              andn    outa,#%0000_0110 ' HX_CTRL_MASK         ' clear control lines to "00", make hole for command
              or      outa,#HX_CMD_READ_SHIFT   ' outa = (outa & !HX_CTRL_MASK) | (HX_CMD_READ_SHIFT)
              add     XMM_Addr,XMM_Len          ' update XMM_Addr before we destroy XMM_Len 
:ReadLoop
              ' clock the strobe line and tell the sram controller to initiate the read and auto inc
              or      outa,HX_STROBE_MASK     ' strobe = 1
              ' data is now on bus, retrieve it...
              mov     outx,ina                  ' pull data from external pins
              shr     outx,#HX_DBUS_BIT_SHIFT   ' shift  data 16 time to the right [23..16] are data pins
              and     outx,#$FF                 ' mask the data to 8-bits
              or      XMM_Temp,outx
              ror     XMM_Temp,#8
              ' finally finish the clocking of the read
              andn    outa,HX_STROBE_MASK       ' strobe = 0
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
XMM_Dst       mov     0-0, XMM_Temp             ' save the result
              ' reset data bus to input before leaving
              andn    outa,HX_OUTPUT_MASK 
              andn    dira,HX_DBUS_MASK
              mov     XMM_Last,XMM_Addr         ' update XMM_Last
XMM_ReadLong_ret
XMM_ReadMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITELONG              
'
' XMM_WriteLong : Write long in source register to XMM at address XMM_Addr.
' On entry:
'    XMM_Addr : (32-bit) address to write (up to 22 bits)
'    XMM_Src  : source of this instruction set to source register
' On exit:
'    XMM_Addr incremented by 4.
'
'
' XMM_WriteMult : Write multiple bytes in source register to XMM at address XMM_Addr.
'                 XMM_Activate should be called at least once before this routine is used.
' On entry:
'    XMM_Addr : address to write (up to 22 bits)
'    XMM_Src  : source of this instruction set to source register
'    XMM_Len  : number of bytes to write (usually 1, 2 or 4)
' On exit:
'    XMM_Addr incremented by 4.
'
XMM_WriteLong
              mov     XMM_Len,#4                ' write 4 bytes
XMM_WriteMult
              call    #XMM_TestAndSetAddr       ' Test address and set if required
XMM_Src       mov     XMM_Temp,0-0              ' save the value
              or      dira,HX_DBUS_MASK         ' place data bus into write mode
              ' command lines should already be in output mode,
              ' so only need to write 2-bit command code for write memory
              andn    outa,#%0000_0110 ' HX_CTRL_MASK         ' clear control lines to "00", make hole for command
              or      outa,#HX_CMD_WRITE_SHIFT  ' outa = (outa & !HX_CTRL_MASK) | (HX_CMD_WRITE_SHIFT)                                          
              add     XMM_Addr,XMM_Len          ' update XMM_Addr before we destroy XMM_Len 
:WriteLoop
              mov     outx,XMM_Temp
              and     outx,#$FF
              shl     outx,#HX_DBUS_BIT_SHIFT   ' shift data into position
              andn    outa,HX_DBUS_MASK         ' outa = (outa & !HX_DBUS_MASK), make hole for data
              or      outa,outx                 ' outa = (outa & !HX_DBUS_MASK) | (r0 << 16)                             
              ' clock the strobe line and tell the sram controller to complete the operation
              or      outa,HX_STROBE_MASK       ' strobe = 1
              andn    outa,HX_STROBE_MASK       ' strobe = 0
              ror     XMM_Temp, #8
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
              ' reset data bus to input before leaving
              andn    outa,HX_OUTPUT_MASK 
              andn    dira,HX_DBUS_MASK
              mov     XMM_Last,XMM_Addr         ' update XMM_Last
XMM_WriteLong_ret
XMM_WriteMult_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_WRITEPAGE              
'
' XMM_WritePage : Write bytes from Hub RAM to XMM RAM
' On entry:
'    XMM_Addr  (32-bit): destination address in sram, 22-bits used
'    Hub_Addr  (32-bit): source address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to write. 
'
XMM_WritePage
              call    #XMM_SetAddr              ' setup ram address
              or      dira,HX_DBUS_MASK         ' place data bus into write mode
              ' command lines should already be in output mode,
              ' so only need to write 2-bit command code for write memory
              andn    outa,#%0000_0110 ' HX_CTRL_MASK         ' clear control lines to "00", make hole for command
              or      outa,#HX_CMD_WRITE_SHIFT  ' outa = (outa & !HX_CTRL_MASK) | (HX_CMD_WRITE_SHIFT)                                          
:WriteLoop
              rdbyte  outx,Hub_Addr
              shl     outx,#HX_DBUS_BIT_SHIFT   ' shift data into position
              andn    outa,HX_DBUS_MASK         ' outa = (outa & !HX_DBUS_MASK), make hole for data
              or      outa,outx                 ' outa = (outa & !HX_DBUS_MASK) | (r0 << 16)                             
              ' clock the strobe line and tell the sram controller to complete the operation
              or      outa,HX_STROBE_MASK       ' strobe = 1
              andn    outa,HX_STROBE_MASK       ' strobe = 0
              add     Hub_Addr,#1               ' inc source address
              add     XMM_Addr,#1               ' inc dest address (XMM address auto increments)
              djnz    XMM_Len, #:WriteLoop      ' repeat while XMM_len <> 0
              ' reset data bus to input before leaving
              andn    outa,HX_OUTPUT_MASK 
              andn    dira,HX_DBUS_MASK
              mov     XMM_Last,XMM_Addr         ' update XMM_Last
XMM_WritePage_ret
              ret
'
#endif          
'
#ifdef NEED_XMM_READPAGE              
'
' XMM_ReadPage : Read bytes from XMM RAM to Hub RAM.
' On Entry: 
'    XMM_Addr  (32-bit): source address in sram, 19-bits used
'    Hub_Addr  (32-bit): destination address in main memory, 16-bits used
'    XMM_Len   (32-bit): number of bytes to read. 
'
XMM_ReadPage
              call    #XMM_SetAddr              ' setup ram address
              andn    dira,HX_DBUS_MASK         ' place data bus into read mode
              ' command lines should already be in output mode,
              ' so only need to write 2-bit command code for read memory
              andn    outa,#%0000_0110 ' HX_CTRL_MASK         ' clear control lines to "00", make hole for command
              or      outa,#HX_CMD_READ_SHIFT   ' outa = (outa & !HX_CTRL_MASK) | (HX_CMD_READ_SHIFT)
:ReadLoop
              ' clock the strobe line and tell the sram controller to initiate the read and auto inc
              or      outa,HX_STROBE_MASK     ' strobe = 1
              mov     outx,ina                  ' read byte from source (XMM)
              shr     outx,#HX_DBUS_BIT_SHIFT
              ' finish the clocking of the read
              andn  outa, HX_STROBE_MASK        ' strobe = 0
              wrbyte  outx,Hub_Addr             ' write byte to destination (Hub)
              add     Hub_Addr,#1               ' inc dest address
              add     XMM_Addr,#1               ' inc src address (XMM address auto increments)
              djnz    XMM_Len,#:ReadLoop        ' repeat while XMM_len <> 0
              ' reset data bus to input before leaving
              andn    outa,HX_OUTPUT_MASK 
              andn    dira,HX_DBUS_MASK
              mov     XMM_Last,XMM_Addr         ' update XMM_Last
XMM_ReadPage_ret
              ret
'
#endif          
'
'XMM_TestAndSetAddr : Setup XMM address if different from last address set.
'                     XMM_Activate should be called at least once before this
'                     routine is used.
'On Entry:
'   XMM_Addr : address to set (up to 22 bits)
'   XMM_Last : last address used
'On Exit:
'   XMM_Addr : preserved
'
'XMM_SetAddr : Setup initial XMM address. XMM_Activate should be called
'              at least once before this routine is used.
'On Entry:
'   XMM_Addr : address to set (up to 22 bits)
'On Exit:
'   XMM_Addr : preserved
'
XMM_TestAndSetAddr
              mov     XMM_Test,XMM_Addr         ' is our current address equal to ...
              sub     XMM_Test,XMM_Last         ' ... our last used XMM address?
              tjz     XMM_Test,#XMM_TestAndSetAddr_ret' yes - no need to set address
XMM_SetAddr
              ' place data bus into output mode
              or      dira,HX_DBUS_MASK
              ' place bits 7 .. 0 of 19-bit address onto data bus
              mov     outx,XMM_Addr
              call    #XMM_SetAddrByte
              or      outa,#HX_CMD_LOADLO_SHIFT ' outa = (outa & !HX_CTRL_MASK) | (HX_CMD_LOADLO_SHIFT)
              ' finally clock the strobe line and tell the sram controller to complete the operation
              or      outa,HX_STROBE_MASK       ' strobe = 1
              andn    outa,HX_STROBE_MASK       ' strobe = 0
              ' place bits 15 .. 8 of 19-bit address onto data bus
              mov     outx,XMM_Addr
              shr     outx,#8                   ' move upper 8-bits into lower 8-bits
              call    #XMM_SetAddrByte
              or      outa,#HX_CMD_LOADHI_SHIFT ' outa = (outa & !HX_CTRL_MASK) | (HX_CMD_LOADHI_SHIFT)
              ' finally clock the strobe line and tell the sram controller to complete the operation
              or      outa,HX_STROBE_MASK       ' strobe = 1
              andn    outa,HX_STROBE_MASK       ' strobe = 0
              ' place upper 19 .. 16 of 19-bit address onto data bus
              mov     outx,XMM_Addr
              shr     outx,#16                  ' move upper 3-bits into lower 3-bits
'             and     outx,#$07                 ' mask lower 3-bits (precaution)
              call    #XMM_SetAddrByte
              or      outa,#HX_CMD_LOADHI_SHIFT ' outa = (outa & !HX_CTRL_MASK) | (HX_CMD_LOADHI_SHIFT)
              ' finally clock the strobe line and tell the sram controller to complete the operation
              or      outa,HX_STROBE_MASK       ' strobe = 1
              andn    outa,HX_STROBE_MASK       ' strobe = 0
XMM_TestAndSetAddr_ret
XMM_SetAddr_ret
              ret

XMM_SetAddrByte
              and     outx,#$FF                 ' mask lower 8-bits
              shl     outx,#HX_DBUS_BIT_SHIFT   ' shift data into position
              andn    outa,HX_DBUS_MASK         ' outa = (outa & !HX_DBUS_MASK), make hole for data
              or      outa,outx                 ' outa = (outa & !HX_DBUS_MASK) | (outc << 16)
              ' command lines should already be in output mode,
              ' so only need to write 2-bit command code for load low address
              andn    outa,#%0000_0110 ' HX_CTRL_MASK       ' clear control lines to "00", make hole for command
XMM_SetAddrByte_ret
              ret
 
'
' XMM variables
'
outx          long    $0
'
XMM_Len       long    $0
XMM_Temp      long    $0
XMM_Last      long    $FFFFFFFF                 ' last Addr used to access XMM
XMM_Test      long    $0
XMM_Addr      long    $0
HUB_Addr      long    $0
'
' 32-bit constants, masks, anything that is greater than 9-bits and can't be represented as an immediate
'
'HX_CTRL_MASK    long      %0000_0000_0000_0000_0000_0000_0000_0110
HX_DBUS_MASK    long      %0000_0000_1111_1111_0000_0000_0000_0000
HX_STROBE_MASK  long      %0100_0000_0000_0000_0000_0000_0000_0000
HX_OUTPUT_MASK  long      %0100_0000_1111_1111_0000_0000_0000_0110
'
'----------------------- End of HX512 XMM Support Routines ---------------------
'
#endif
'
