'
' Target-specific PASM for the BlackCat Debugger ...
'
{
'
' Should really include this, but it changes too many offsets ...
'
OBJ
  Common : "Catalina_Common.spin"
'
' So instead we just include the necessary constants below,
' which must match the ones in Catalina_Common.spin ...
'
}

CON
  DEBUG_BREAK   = $7F10           ' Non-zero when at breakpoint and executing debug support code
  DEBUG_ADDR    = $7F14           ' Holds address in kernel for a host requested read or write
  DEBUG_OUT     = $7F18           ' Holds information written out from kernel space
  DEBUG_IN      = $7F1C           ' Holds information to be written into kernel space
  DEBUG_FLAG    = $7F20           ' Indicates read operation requested when non-zero
  DEBUG_OVERLAY = $53             ' Cog location to overlay debug code
  DEBUG_VECTORS = $1eb            ' Cog location to overlay hub vectors (last 5 longs!!!)
  DEBUG_MARKER  = $2b             ' Cog location to write to DEBUG_BREAK at breakpoint                    
  DEBUG_END     = $7F2F           ' Last hub address reseved for debugger - next address is $7F30

  VECTOR_SIZE   = 5               ' 5  Longs of vectors

#ifdef LARGE
  OVERLAY_SIZE  = 28              ' 28 Longs for XMM LARGE
#elseifdef SMALL
  OVERLAY_SIZE  = 28              ' 28 Longs for XMM SMALL
#else
  OVERLAY_SIZE  = 17              ' 17 Longs for LMM
#define LMM_OVERLAY
#endif

'
' Catalina Code

DAT ' code segment

'
' C_arg_setup : setup argc in r3 and argv in r2
'
C_arg_setup
 jmp #LODA                 ' point to argv address
 long @C_argc_addr
#ifdef LARGE
 jmp #RLNG
#elseifdef SMALL
 jmp #RLNG
#else
 rdlong BC,RI
#endif 
 rdword r3,BC              ' load argc
 add BC,#2
 rdword r2,BC              ' load argv
 jmp #RETN                 ' done

C_argc_addr
 long $7F34                ' must match value in Catalina_Common_Input.spin

'
' C_debug_init : load debug code into kernel, set up debug vectors, then BREAK
'
C_debug_init
 mov r2,#OVERLAY_SIZE      ' load overlay code ...
 mov r0,#DEBUG_OVERLAY     ' ... from C_debug_overlay ...
 jmp #LODA                 ' ... to replace ...
 long @C_debug_overlay     ' ... cog ... 
 mov r1,RI                 ' ... initialization code
 jmp #CALA                 ' do ...
 long @C_overlay_loader    ' ... the load
 mov r2,#VECTOR_SIZE       'load vectors ...
 mov r0,#DEBUG_VECTORS     ' ... from C_debug_vectors ...
 jmp #LODA                 ' ... to ... 
 long @C_debug_vectors     ' ... cog ...
 mov r1,RI                 ' ... vector table
 jmp #CALA                 ' do ...
 long @C_overlay_loader    ' ... the load
 jmp #LODA                 ' indicate ...
 long @C_debug_addr_vector ' ... special ...
#ifdef LARGE
 jmp #RLNG
#elseifdef SMALL
 jmp #RLNG
#else
 rdlong BC,RI              ' ... breakpoint ...
#endif
 neg r0,#1                 ' ... by setting ...
 wrlong r0,BC              ' ... DEBUG_ADDR to -1
 jmp #INIT                 ' !!! BREAKPOINT !!!
 jmp #RETN                 ' done

'
' C_overlay_loader : load code into kernel
'   On entry:
'     r0 = first register in cog to load
'     r1 = address of first long in hub RAM to load
'     r2 = size in longs
'   On exit:
'     r0,r1,r2,r3,r4 lost
'
C_overlay_loader

#ifdef LMM_OVERLAY
 jmp #LODA                 ' set up ...
 long @C_overlay_loader_2  ' ... initial ...
 rdlong r3,RI              ' ... destination ...
 movd r3,r0                ' ... register ...
 wrlong r3,RI              ' ... to be loaded ...
C_overlay_loader_1
 sub r2,#1 wz,wc           ' if all ...
 jmp #BR_B                 ' ... data loaded ...
 long @C_overlay_loader_3  ' ... then exit
C_overlay_loader_2
 rdlong 0-0,r1             ' load a long into the cog
 add r1,#4                 ' point to next long
 jmp #LODA                 ' increment ...
 long @C_overlay_loader_2  ' ...
 mov r4,RI                 ' ...
 rdlong r0,r4              ' ...
 jmp #LODA                 ' ... destination ...
 long @C_d_inc             ' ... 
 rdlong r3,RI              ' ...
 add r0,r3                 ' ... 
 wrlong r0,r4              ' ... register
 jmp #JMPA                 ' load ...
 long @C_overlay_loader_1  ' ... more longs
#else
 jmp #LODA                 ' set up ...
 long @C_overlay_loader_2  ' ... initial ...
 jmp #RLNG                 '
 movd BC,r0                ' ... register ...
 jmp #WLNG                 '
C_overlay_loader_1
 sub r2,#1 wz,wc           ' if all ...
 jmp #BR_B                 ' ... data loaded ...
 long @C_overlay_loader_3  ' ... then exit
 mov RI,r1                 '
 jmp #RLNG                 '
C_overlay_loader_2
 mov 0-0,BC                '
 add r1,#4                 ' point to next long
 jmp #LODA                 ' increment ...
 long @C_overlay_loader_2  ' ...
 jmp #RLNG                 '
 mov r0,BC                 '
 jmp #LODA                 ' ... destination ...
 long @C_d_inc             ' ... 
 jmp #RLNG                 '
 add BC,r0                 ' ... 
 jmp #LODA                 ' 
 long @C_overlay_loader_2  ' ...
 jmp #WLNG                 '
 jmp #JMPA                 ' load ...
 long @C_overlay_loader_1  ' ... more longs
#endif
C_overlay_loader_3         ' done
 jmp #RETN

'
' C_debug_overlay : debug code to load into kernel
'
' The code was previously assembled, using the following constant definitions:
'  
'  opAddr     = $00    ' cog address 0 used as register (normally reserved for POD)
'  temp       = $01    ' cog address 1 used as register (normally reserved for POD)
'  breakFlag  = $1eb   ' vector to be set to Common#DEBUG_BREAK
'  addr       = $1ec   ' vector to be set to Common#DEBUG_ADDR
'  kernelOut  = $1ed   ' vector to be set to Common#DEBUG_OUT
'  kernelIn   = $1ee   ' vector to be set to Common#DEBUG_IN
'  opFlag     = $1ef   ' vector to be set to Common#DEBUG_FLAG
'
' The LMM code is common to all platforms, but the XMM code is platform
' specififc because of the different sizes of the XMM support code.
'
'=====================================================================================================================
' HERE IS THE LMM CODE (FOR REFERENCE - BINARY VERSIONS INCLUDED BELOW):
'
'                         org     DEBUG_OVERLAY
' brkpCode                wrlong  DEBUG_MARKER,breakFlag  '1 Let the host know that we are at a breakpoint              
' readFlags               rdlong  temp,breakFlag          '2 Test for host presenting continuation instruction          
'                         tjz     temp,#continue          '3 ..go here to replace breakpoint instruction and run        
'                         rdlong  opAddr,addr             '4 opAddr now holds the address for a kernel read/write       
'                         tjz     opAddr,#readFlags       '5 ..but address zero means no read/write is requested        
'                         movd    kernelRead,opAddr       '6 opAddr contains the kernel address to be read ...          
'                         movd    kernelWrite,opAddr      '7 ... or written (don't know which yet,so set up both)       
'                         rdlong  temp,opFlag             '8 Is it a read?                                              
'                         tjz     temp,#kernelWrite       '9 no - write to kernel                                       
' kernelRead              wrlong  0-0,kernelOut           '10 Write from cog location to DEBUG_OUT.                      
'                         jmp     #finish                 '11                                                            
' kernelWrite             rdlong  0-0,kernelIn            '12 Read from DEBUG_IN to cog location.                        
' finish                  mov     temp,#0                 '13                                                            
'                         wrlong  temp,addr               '14 Write a zero to DEBUG_ADDR so host sees operation complete.
'                         jmp     #readFlags              '15 wait for continuation or next command.                     
' continue                rdlong  LMM_1_OFF,kernelIn      '16 Read replacement instruction from  DEBUG_IN                
'                         jmp     #LMM_1_OFF              '17 ..and restart the kernel
'
'=======================================================================================================================
' HERE IS THE XMM CODE (FOR REFERENCE - BINARY VERSIONS INCLUDED BELOW):
'
'                         org     DEBUG_OVERLAY
' brkpCode                wrlong  DEBUG_MARKER,breakFlag  '1 Let the host know that we are at a breakpoint                
' readFlags               rdlong  temp,breakFlag          '2 Test for host presenting continuation instruction            
'                         tjz     temp,#continue          '3 ..go here to replace breakpoint instruction and run          
'                         rdlong  XMM_Addr,addr           '4 XMM_Addr holds the address to read/write (cog or XMM addr)   
'                         tjz     XMM_Addr,#readFlags     '5 zero means no read/write is requested                        
'                         movd    kernelRead,XMM_Addr     '6 for kernel read, XMM_Addr contains the cog address to read   
'                         movd    kernelWrite,XMM_Addr    '7 for kernel write,  XMM_Addr contains the cog address to write
'                         rdlong  temp,opFlag             '8 if command ...                                               
'                         tjnz    temp,#decodeRead        '9 ... is not kernel write, then decode further                 
' kernelWrite             rdlong  0-0,kernelIn            '10 write to cog address from DEBUG_IN                          
'                         jmp     #finish                 '11 ... to cog address                                          
' decodeRead              djnz    temp,#decodeXMM         '12 read from kernel?                                           
' kernelRead              wrlong  0-0,kernelOut           '13 yes - write to DEBUG_OUT ...                                
'                         jmp     #finish                 '14 ... from cog address                                        
' decodeXMM               sub     XMM_Addr,CS             '15 correct for moved code segment                              
'                         djnz    temp,#xmmRead           '16 write to XMM?                                               
' xmmWrite                rdlong  temp,kernelIn           '17 write ...                                                   
'                         movs    XMM_Src,#temp           '18 ... the value                                               
'                         call    #XMM_WriteLong          '19 ... in kernelIn                                             
'                         jmp     #finish                 '20 ... from DEBUG_IN                                           
' xmmRead                 movd    XMM_Dst,#temp           '21 read ...                                                    
'                         call    #XMM_ReadLong           '22 ... the value ...                                           
'                         wrlong  temp,kernelOut          '23 ... to kernelOut                                            
' finish                  mov     temp,#0                 '24 write a zero ...                                            
'                         wrlong  temp,addr               '25 ... to DEBUG_ADDR to indicate command completed             
'                         jmp     #readFlags              '26 wait for continuation, or next command                      
' continue                rdlong  LMM_1_OFF,kernelIn      '27 read replacement instruction from DEBUG_IN ...              
'                         jmp     #LMM_1_OFF              '28 ... and restart the kernel processing                       
'
'=======================================================================================================================

C_debug_overlay  ' binary overlay starts here

#ifdef LMM_OVERLAY

' ALL LMM PLATFORMS USE THE SAME OVERLAY CODE:
'
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c0262 ' 3
 long $08bc01ec ' 4
 long $ec7c0054 ' 5
 long $54bcb800 ' 6
 long $54bcbc00 ' 7
 long $08bc03ef ' 8
 long $ec7c025e ' 9
 long $083c01ed ' 10
 long $5c7c005f ' 11
 long $08bc01ee ' 12
 long $a0fc0200 ' 13
 long $083c03ec ' 14
 long $5c7c0054 ' 15
 long $08bccfee ' 16
 long $5c7c0067 ' 17

#elseifdef TRIBLADEPROP

#ifdef CPU_1

#ifdef LARGE
' TRIBLADEPROP CPU_1 LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bfa1ec ' 4
 long $ec7fa054 ' 5
 long $54bcbfd0 ' 6
 long $54bcb9d0 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bfa032 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff4e01 ' 18
 long $5cff69a5 ' 19
 long $5c7c006a ' 20
 long $54ff4601 ' 21
 long $5cff499a ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' TRIBLADEPROP CPU_1 SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf55ec ' 4
 long $ec7f5454 ' 5
 long $54bcbfaa ' 6
 long $54bcb9aa ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf5432 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff0201 ' 18
 long $5cff1d7f ' 19
 long $5c7c006a ' 20
 long $54fefa01 ' 21
 long $5cfefd74 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#elseifdef CPU_3

  ERROR : TRIBLADEPROP HAS NO XMM RAM ON CPU_3 !!!

#else

#ifdef LARGE
' TRIBLADEPROP CPU_2 LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bfb5ec ' 4
 long $ec7fb454 ' 5
 long $54bcbfda ' 6
 long $54bcb9da ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bfb432 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff5001 ' 18
 long $5cff6ba6 ' 19
 long $5c7c006a ' 20
 long $54ff4801 ' 21
 long $5cff4b9a ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' TRIBLADEPROP CPU_2 SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf69ec ' 4
 long $ec7f6854 ' 5
 long $54bcbfb4 ' 6
 long $54bcb9b4 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf6832 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff0401 ' 18
 long $5cff1f80 ' 19
 long $5c7c006a ' 20
 long $54fefc01 ' 21
 long $5cfeff74 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#endif

#elseifdef DRACBLADE

#ifdef LARGE
' DRACBLADE LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bfc3ec ' 4
 long $ec7fc254 ' 5
 long $54bcbfe1 ' 6
 long $54bcb9e1 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bfc232 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff4c01 ' 18
 long $5cff65a4 ' 19
 long $5c7c006a ' 20
 long $54ff4401 ' 21
 long $5cff4795 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' DRACBLADE SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf77ec ' 4
 long $ec7f7654 ' 5
 long $54bcbfbb ' 6
 long $54bcb9bb ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf7632 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff0001 ' 18
 long $5cff197e ' 19
 long $5c7c006a ' 20
 long $54fef801 ' 21
 long $5cfefb6f ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#elseifdef MORPHEUS

#ifdef CPU_1

#ifdef LARGE
' MORPHEUS CPU_1 LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bfc3ec ' 4
 long $ec7fc254 ' 5
 long $54bcbfe1 ' 6
 long $54bcb9e1 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bfc232 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff6601 ' 18
 long $5cff77ae ' 19
 long $5c7c006a ' 20
 long $54ff5401 ' 21
 long $5cff5b9e ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' MORPHEUS CPU_1 SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf77ec ' 4
 long $ec7f7654 ' 5
 long $54bcbfbb ' 6
 long $54bcb9bb ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf7632 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff1a01 ' 18
 long $5cff2b88 ' 19
 long $5c7c006a ' 20
 long $54ff0801 ' 21
 long $5cff0f78 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#else

#ifdef LARGE
' MORPHEUS CPU_2 LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf91ec ' 4
 long $ec7f9054 ' 5
 long $54bcbfc8 ' 6
 long $54bcb9c8 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf9032 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff4a01 ' 18
 long $5cff63a3 ' 19
 long $5c7c006a ' 20
 long $54ff4201 ' 21
 long $5cff4595 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' MORPHEUS CPU_2 SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf45ec ' 4
 long $ec7f4454 ' 5
 long $54bcbfa2 ' 6
 long $54bcb9a2 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf4432 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50fefe01 ' 18
 long $5cff177d ' 19
 long $5c7c006a ' 20
 long $54fef601 ' 21
 long $5cfef96f ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#endif

#elseifdef RAMBLADE

#ifdef LARGE
' RAMBLADE LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf79ec ' 4
 long $ec7f7854 ' 5
 long $54bcbfbc ' 6
 long $54bcb9bc ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf7832 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff5201 ' 18
 long $5cff69a6 ' 19
 long $5c7c006a ' 20
 long $54ff4801 ' 21
 long $5cff4b97 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' RAMBLADE SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf2dec ' 4
 long $ec7f2c54 ' 5
 long $54bcbf96 ' 6
 long $54bcb996 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf2c32 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff0601 ' 18
 long $5cff1d80 ' 19
 long $5c7c006a ' 20
 long $54fefc01 ' 21
 long $5cfeff71 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#elseifdef C3

#ifdef LARGE
' C3 LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bfc3ec ' 4
 long $ec7fc254 ' 5
 long $54bcbfe1 ' 6
 long $54bcb9e1 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bfc232 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff6001 ' 18
 long $5cff37ac ' 19
 long $5c7c006a ' 20
 long $54ff5401 ' 21
 long $5cff37a0 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' C3 SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf77ec ' 4
 long $ec7f7654 ' 5
 long $54bcbfbb ' 6
 long $54bcb9bb ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf7632 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff1401 ' 18
 long $5cfeeb86 ' 19
 long $5c7c006a ' 20
 long $54ff0801 ' 21
 long $5cfeeb7a ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#else

#ifdef LARGE
' HX512 (HYDRA/HYBRID) LARGE XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bfc3ec ' 4
 long $ec7fc254 ' 5
 long $54bcbfe1 ' 6
 long $54bcb9e1 ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bfc232 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff5c01 ' 18
 long $5cff7fac ' 19
 long $5c7c006a ' 20
 long $54ff4e01 ' 21
 long $5cff5798 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#else
' HX512 (HYDRA/HYBRID) SMALL XMM:
 long $083c57eb ' 1
 long $08bc03eb ' 2
 long $ec7c026d ' 3
 long $08bf77ec ' 4
 long $ec7f7654 ' 5
 long $54bcbfbb ' 6
 long $54bcb9bb ' 7
 long $08bc03ef ' 8
 long $e87c025e ' 9
 long $08bc01ee ' 10
 long $5c7c006a ' 11
 long $e4fc0261 ' 12
 long $083c01ed ' 13
 long $5c7c006a ' 14
 long $84bf7632 ' 15
 long $e4fc0267 ' 16
 long $08bc03ee ' 17
 long $50ff1001 ' 18
 long $5cff3386 ' 19
 long $5c7c006a ' 20
 long $54ff0201 ' 21
 long $5cff0b72 ' 22
 long $083c03ed ' 23
 long $a0fc0200 ' 24
 long $083c03ec ' 25
 long $5c7c0054 ' 26
 long $08bce5ee ' 27
 long $5c7c0072 ' 28
#endif

#endif

'
' C_d_inc : constant used to increment instruction destination register
'
C_d_inc
 long 1<<9

'
' C_debug_vectors : hub ram vectors to load into kernel 
' 
' Note: DEBUG_ADDR must be set to -1 to indicate first breakpoint
'
C_debug_vectors
 long DEBUG_BREAK
C_debug_addr_vector
 long DEBUG_ADDR 
 long DEBUG_OUT  
 long DEBUG_IN
 long DEBUG_FLAG 
